#define LUTIL
/* util.c */
/* some common functions used in loading images, but also generic utility functions */

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <zlib.h>
#include <limits.h>
#include "loadimg.h"

static char *tmpDir=NULL;
static char *gzipCommand=NULL;
static char *unzipCommand=NULL;
#ifdef __MINGW32__
static double drand48() 
{
  return (double)(rand()) / (double)(RAND_MAX);
}
#endif

DLUTIL void llibSetTmpDir(char * value)
{ tmpDir = strdup(value); }

DLUTIL void llibSetGzipCommand(char * value)
{ gzipCommand = strdup(value); }

DLUTIL void llibSetUnzipCommand(char * value)
{ unzipCommand = strdup(value); }

DLUTIL char *llibGetTmpDir(void)
{ return(tmpDir); }

DLUTIL char *llibGetGzipCommand(void)
{ return(gzipCommand); }

DLUTIL char *llibGetUnzipCommand(void)
{ return(unzipCommand); }

/* removes keyword identifier in a line of settings; delimiter is '=' */
DLUTIL char *removeHead(char *inpstr)
{
 int i=0;
 int len = strlen(inpstr);

 while ((i < len) && (inpstr[i]!='='))
  i++;
 i++;
 return (&inpstr[i]);
}

/* used to remove last character, like /n */
DLUTIL char *removeLast(char *textString)
{
 int length;

 length = strlen(textString);
 if (length)
  textString[length-1] = 0;
 return(textString);
}

/* this table and following function are repeated elsewhere in the code */
/* which calls this library, but we keep another "copy" to have self-contained code */
static unsigned char transTable[256];

/* remaps the data passed according to values in transTable */
static void remapData(unsigned char *data, int length)
{
 unsigned char *limit;

 limit = &data[length];
 data--;
 while (++data<limit)
  *data=transTable[*data];
}

/* reverse and, if necessary, negate bytes in the passed data field */
DLUTIL void reverseNegateBytes(unsigned char *data, int length, int negate)
{
 int i,j;

 for (i=0;i<256; i++) /* prepare table for reversing bytes */
  {
   transTable[i]=0;
   for (j=0; j<8; j++)
    if (i&(1<<j))
     transTable[i] = (unsigned char) (transTable[i] + (128>>j));
   if (negate)
    transTable[i] = (unsigned char) ~transTable[i];
  }
 remapData(data, length);
}

/* reimplementation of fgets which reads MSDOS, UNIX and DOS ASCII files (end of line madess) */
DLUTIL char *fgetsDosMac(char *inpStr, int limit, FILE *fp)
{
 int i=0, c=0;
 
 if (!feof(fp))
  c=fgetc(fp);
 /* avoid generating empty lines on CRLF */
 if (((c==0xD) || (c==0xA)) && (!feof(fp)))
  c=fgetc(fp);
 while ( (!feof(fp)) && (i<limit) && (c!=0xD) && (c!=0xA) )
  {
   inpStr[i++] = (char) c;
   c = fgetc(fp);
  }
 inpStr[min(i, limit-1)]=0; /* null terminate it */
 return(i ? inpStr : NULL);
}

/* simple ascii version of tolower, since on Windows it does not work properly, and for our needs converting A-Z to a-z is enough */
DLUTIL char tolowerAZ(char in)
{
 if ((in>='A')&&(in<='Z'))
  return(in+('a'-'A'));
 else
  return(in);
}

/* convert to lowercase*/
DLUTIL void stringToLower(char *line)
{
 int i, length;

 length = strlen(line);
 if (length)
  for (i=0; i<length; i++)
   line[i] = (char) tolowerAZ(line[i]);
}

/* use small tables, so we avoid trashing cache */
#define MAX_RAND 2048
#define MAX_RAND_MASK 0x7ff
/* unified function to generate real view image for both screen and print in any depth */
/* there are no implicit parameters, can be extraced and used as generic function */
/* some repetition of code, but can not afford a switch within inner loop */
DLUTIL void renderRealImg(unsigned char *dstData, int dstBPL, int bytesPerPixel,
 int xpos, int ypos, int width, int height, int firstBottomPix, int dstxdpi, int dstydpi,
 unsigned char *srcData, int srcBPL, int srcWid, int srcHei, int srcImageType,
 unsigned char *srcCmap, int srcxdpi, int srcydpi, float zoomFactor)
{
 unsigned short xrandList[MAX_RAND], yrandList[MAX_RAND];
 int xfactor, yfactor, x, y;
 unsigned char *dst8=NULL;
 unsigned short *dst16=NULL;
 unsigned int *dst32=NULL;
 unsigned int srcy, srcx; /* on big zoom-outs it can go out of bounds, but it must not be negative (modulo %!)*/
 int i, j, valr, valg, valb;
 unsigned int randInd;
 unsigned char *srcPtr[MAX_LOAD_IMAGE], *dataPtr;

//printf("render real from pos %d,%d %dx%d image %dx%d zoom %.3f\n", xpos, ypos, width, height, srcWid, srcHei, zoomFactor);

 xfactor = (int) ((8192.0 * srcxdpi ) / (1.0 * dstxdpi * zoomFactor));
 yfactor = (int) ((8192.0 * srcydpi ) / (1.0 * dstydpi * zoomFactor));

 /* tables are small, so calculate this every time */
 for (x=0; x<MAX_RAND; x++)
  xrandList[x] = (unsigned short) (drand48()*xfactor);
 for (x=0; x<MAX_RAND; x++)
  yrandList[x] = (unsigned short) (drand48()*yfactor);

 /* prepare a table for inverse bottom up rendering and bpl calculation */
 for (y=0; y<srcHei; y++)
  srcPtr[y] = &srcData[srcBPL*(srcHei - y -1)];
 /* render from bottom  up! */
 for (y=ypos+height-1; y>=ypos; y--)
  {
   randInd = (unsigned int) (drand48()*MAX_RAND); /* restart index on each line, to avoid cyclic artefacts */
   srcy = (firstBottomPix - y)*yfactor;
   dst8 = dstData + (y-ypos)*dstBPL;
   dst16 = (unsigned short *) (dstData + (y-ypos)*dstBPL);
   dst32 = (unsigned int *) (dstData + (y-ypos)*dstBPL);
   switch (bytesPerPixel)
    {
     case 1:
      if (srcImageType==LOAD_BW)
       for (x = 0; x < width; x++)
	{
	 j = ((((x+xpos)*xfactor + xrandList[(randInd++)&MAX_RAND_MASK]) >> 13)%srcWid);
	 dataPtr = srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] + (j>>3);
//	 *dst8++ = (unsigned char) ((*dataPtr&(128>>(j&7))) ? 0 : 1);
	 *dst8++ = (unsigned char) ((*dataPtr&(128>>(j&7))) ? 255 : 254);
	}
      else /* only LOAD_CMAP mode supported here */
       for (x = 0; x < width; x++)
	{
	 *dst8++ = *(srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] +
	  ((((x+xpos)*xfactor + xrandList[randInd&MAX_RAND_MASK]) >> 13)%srcWid));
	 randInd++;
	}
      break;  
     case 2:
      switch (srcImageType)
       {
        case LOAD_BW:
	 for (x = 0; x < width; x++)
	  {
	   valr=0;
	   srcx = (x+xpos)*xfactor;
	   for (i=32; i; i--)
            {
	     j = (((srcx + xrandList[(randInd++)&MAX_RAND_MASK]) >> 13)%srcWid);
	     dataPtr = srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] + (j>>3);
	     if (!(*dataPtr&(128>>(j&7))))
	      valr+=255;
   	    }
           valb = valr>>8; valg = valr>>7; valr>>=8; /* the order of these two operations matters */
	   *dst16++ = (unsigned short)((valr<<11) | (valg<<5) | valb); 
	  }
	 break;
	case LOAD_CMAP:
	 for (x = 0; x < width; x++)
	  {
	   valr=valg=valb=0;
	   srcx = (x+xpos)*xfactor;
	   for (i=32; i; i--)
            {
	     dataPtr = &srcCmap[3* *(srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] +
	      (((srcx + xrandList[randInd&MAX_RAND_MASK]) >> 13)%srcWid))];
	     randInd++;
	     valr+= *dataPtr++; valg+=*dataPtr++; valb+=*dataPtr;
   	    }
	   valr>>=8; valg>>=7; valb>>=8;
	   *dst16++ = (unsigned short)((valr<<11) | (valg<<5) | valb); 
	  }
	 break;
        case LOAD_TRUE_COL:
	 for (x = 0; x < width; x++)
	  {
	   valr=valg=valb=0;
	   srcx = (x+xpos)*xfactor;
	   for (i=32; i; i--)
            {
	     dataPtr =  srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] +
	      3*(((srcx + xrandList[randInd&MAX_RAND_MASK]) >> 13)%srcWid);
	     randInd++; valr+= *dataPtr++; valg+=*dataPtr++; valb+=*dataPtr;
   	    }
	   valr>>=8; valg>>=7; valb>>=8;
	   *dst16++ = (unsigned short)((valr<<11) | (valg<<5) | valb); 
	  }
	 break;
       }
      break;  
     case 3:
      switch (srcImageType)
       {
        case LOAD_BW:
	 for (x = 0; x < width; x++)
	  {
	   valr=0;
	   srcx = (x+xpos)*xfactor;
	   for (i=32; i; i--)
            {
	     j = (((srcx + xrandList[(randInd++)&MAX_RAND_MASK]) >> 13)%srcWid);
	     dataPtr = srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] + (j>>3);
	     if (!(*dataPtr&(128>>(j&7))))
	      valr+=255;
   	    }
	   valr>>=5;
	   *dst8++ = (unsigned char)valr;
	   *dst8++ = (unsigned char)valr;
	   *dst8++ = (unsigned char)valr; 
	  }
	 break;
	case LOAD_CMAP:
	 for (x = 0; x < width; x++)
	  {
	   valr=valg=valb=0;
	   srcx = (x+xpos)*xfactor;
	   for (i=32; i; i--)
            {
	     dataPtr = &srcCmap[3* *(srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] +
	      (((srcx + xrandList[randInd&MAX_RAND_MASK]) >> 13)%srcWid))];
	     randInd++; valr+= *dataPtr++; valg+=*dataPtr++; valb+=*dataPtr;
   	    }
	   *dst8++ = (unsigned char)(valr>>5);
	   *dst8++ = (unsigned char)(valg>>5);
	   *dst8++ = (unsigned char)(valb>>5);
	  }
	 break;
        case LOAD_TRUE_COL:
	 for (x = 0; x < width; x++)
	  {
	   valr=valg=valb=0;
	   srcx = (x+xpos)*xfactor;
	   for (i=32; i; i--)
            {
	     dataPtr =  srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] +
	      3*(((srcx + xrandList[randInd&MAX_RAND_MASK]) >> 13)%srcWid);
	     randInd++; valr+= *dataPtr++; valg+=*dataPtr++; valb+=*dataPtr;
   	    }
	   *dst8++ = (unsigned char)(valr>>5);
	   *dst8++ = (unsigned char)(valg>>5);
	   *dst8++ = (unsigned char)(valb>>5); 
	  }
	 break;
       }
      break;  
     case 4:
      switch (srcImageType)
       {
        case LOAD_BW:
	 for (x = 0; x < width; x++)
	  {
	   valr=0;
	   srcx = (x+xpos)*xfactor;
	   for (i=32; i; i--)
            {
	     j = (((srcx + xrandList[(randInd++)&MAX_RAND_MASK]) >> 13)%srcWid);
	     dataPtr = srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] + (j>>3);
	     if (!(*dataPtr&(128>>(j&7))))
	      valr+=255;
   	    }
	   valr>>=5;
	   *dst32++ = (valr<<16) | (valr<<8) | valr; 
	  }
	 break;
	case LOAD_CMAP:
	 for (x = 0; x < width; x++)
	  {
	   valr=valg=valb=0;
	   srcx = (x+xpos)*xfactor;
	   for (i=32; i; i--)
            {
	     dataPtr = &srcCmap[3* *(srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] +
	      (((srcx + xrandList[randInd&MAX_RAND_MASK]) >> 13)%srcWid))];
	     randInd++;
	     valr+= *dataPtr++; valg+=*dataPtr++; valb+=*dataPtr;
   	    }
	   valr>>=5; valg>>=5; valb>>=5;
	   *dst32++ = (valr<<16) | (valg<<8) | valb; 
	  }
	 break;
        case LOAD_TRUE_COL:
	 for (x = 0; x < width; x++)
	  {
	   valr=valg=valb=0;
	   srcx = (x+xpos)*xfactor;
	   for (i=32; i; i--)
            {
	     dataPtr =  srcPtr[((srcy + yrandList[randInd&MAX_RAND_MASK]) >> 13)%srcHei] +
	      3*(((srcx + xrandList[randInd&MAX_RAND_MASK]) >> 13)%srcWid);
	     randInd++; valr+= *dataPtr++; valg+=*dataPtr++; valb+=*dataPtr;
   	    }
	   valr>>=5; valg>>=5;
	   *dst32++ = (valr<<16) | (valg<<8) | (valb>>5); 
	  }
	 break;
       }
      break;  
    } 
  }
}

/* generate a list of points which are randomly distributed across the image */
/* but with similar distance to other points, with random oscillation */
/* extremely high values of numPoints should be avoided, as it becomes very slow */
/* caller must test the result and free the allocated array */
DLUTIL unsigned short *selfAvoidTable(int numPoints, int wid, int hei)
{
 int i,j,k;
 unsigned short *dropXY, *dropXYptr;
 int numCandidates = 60;
 int tryX, tryY;
 int dx1, dx2, dx3;
 int dy1, dy2, dy3;
 int minDist;
 int maxMinDist;
 int bestTryX =0;
 int bestTryY =0;
 
 dropXY = malloc(sizeof(unsigned short)*numPoints*2);
 if (!dropXY)
  return(dropXY);

 for (i=0; i<numPoints; i++)
  {
   maxMinDist = 0;
   for (j=0; j<numCandidates; j++)
    {
     tryX = rand()%wid;
     tryY = rand()%hei;
     minDist=INT_MAX;
     dropXYptr = dropXY;
     for (k=0; k<i; k++)
      {
       // distance must work with wrap around
       dx1 = tryX - *dropXYptr++;
       dx2 = dx1+wid;
       dx3 = dx1-wid;
       dx1 *= dx1;
       dx2 *= dx2;
       if (dx2<dx1)
	dx1=dx2;
       dx3 *= dx3;
       if (dx3<dx1)
	dx1=dx3;

       dy1 = tryY - *dropXYptr++;
       dy2 = dy1+hei;
       dy3 = dy1-hei;
       dy1 *= dy1;
       dy2 *= dy2;
       if (dy2<dy1)
	dy1=dy2;
       dy3 *= dy3;
       if (dy3<dy1)
	dy1=dy3;
       if ((dx1+dy1)<minDist)
	{
	 minDist = dx1+dy1;
         if (minDist<maxMinDist)
	  k=i; // bailout
	}
      }
     if (maxMinDist<minDist)
      {
       maxMinDist = minDist;
       bestTryX = tryX;
       bestTryY = tryY;
      }
    }
   dropXY[i<<1] = (unsigned short) bestTryX;
   dropXY[(i<<1)+1] = (unsigned short) bestTryY;
  }
 return(dropXY);
}

#define MAX_CELLS 256
/* stretch the contrast on cmap or true color images. */
/* on true color images modify the image, on colormap only modify the colormap */
DLUTIL void contrastStretch(unsigned char *imageData, int imageBPL, int imageType, int wid, int hei, unsigned char *cmap)
{
 unsigned char minRGB[3] = {255, 255, 255}, maxRGB[3] = {0, 0, 0};
 int diffRGB[3];
 int i,j, x,y;
 unsigned char *data;

 if (imageType==LOAD_CMAP)
  {
   int imageHist[MAX_CELLS];

   memset(imageHist, 0, MAX_CELLS*sizeof(int));
   data = imageData;
   for (i=wid*hei; i; i--)
    imageHist[*data++]++;

   for (i=0; i<MAX_CELLS; i++)
    if (imageHist[i])
     for (j=0; j<3; j++)
      {
       if (cmap[i*3+j]<minRGB[j])
	minRGB[j] = cmap[i*3+j];
       if (cmap[i*3+j]>maxRGB[j])
	maxRGB[j] = cmap[i*3+j];
     }
  }
 else
  for (y=0; y<hei; y++)
   {
    data = &imageData[y*imageBPL];
    for (x=0; x<wid; x++)
     {
       if (*data<minRGB[0])
        minRGB[0] = *data;
       if (*data>maxRGB[0])
        maxRGB[0] = *data;
       data++;
       if (*data<minRGB[1])
        minRGB[1] = *data;
       if (*data>maxRGB[1])
        maxRGB[1] = *data;
       data++;
       if (*data<minRGB[2])
        minRGB[2] = *data;
       if (*data>maxRGB[2])
        maxRGB[2] = *data;
       data++;
     }
   }
 for (i=0; i<3; i++)
  diffRGB[i]=maxRGB[i]-minRGB[i];
 if ( (diffRGB[0]<=0) || (diffRGB[1]<=0) || (diffRGB[2]<=0) )
  return;
 if (imageType==LOAD_CMAP)
  for (i=0; i<MAX_CELLS; i++)
   for (j=0; j<3; j++)
    cmap[i*3+j] = (unsigned char) ((cmap[i*3+j]-minRGB[j]) * 255.0/diffRGB[j]);
 else
  {
   unsigned char remap[3][256];

   for (j=0; j<3; j++)
    for (i=0; i<MAX_CELLS; i++)
     remap[j][i] = (unsigned char) ((i-minRGB[j]) * 255.0/diffRGB[j]);

   for (y=0; y<hei; y++)
    {
     data = &imageData[y*imageBPL];
     for (x=0; x<wid; x++)
      {
       *data = remap[0][*data]; data++;
       *data = remap[1][*data]; data++;
       *data = remap[2][*data]; data++;
      }
    }
  }
}

/* interpret background as white or check and convert RGBA to RGB in memory, also realloc memory */
DLUTIL unsigned char *collapseRGBA(unsigned char *data, int wid, int hei, int *bpl, int doCheck)
{
 int x, y;
 unsigned char *srcPtr, *dstPtr, checkPoint;
 int invertAlpha;

 dstPtr = data;
 for (y=0; y<hei; y++)
  {
   srcPtr = &data[y * *bpl];
   if (doCheck)
    for (x=0; x<wid; x++)
     {
      checkPoint = (((x>>3)+(y>>3))&1) ? DARK_CHECK : LIGHT_CHECK;
      if (srcPtr[3])
       {
	if (srcPtr[3]==255)
	 {
	  *dstPtr++ = srcPtr[0];
	  *dstPtr++ = srcPtr[1];
	  *dstPtr++ = srcPtr[2];
	 }
	else
	 {
	  invertAlpha = ((int)255-srcPtr[3])*checkPoint;
	  *dstPtr++ = (unsigned char)((invertAlpha + (int)srcPtr[0]*srcPtr[3])>>8);
	  *dstPtr++ = (unsigned char)((invertAlpha + (int)srcPtr[1]*srcPtr[3])>>8);
	  *dstPtr++ = (unsigned char)((invertAlpha + (int)srcPtr[2]*srcPtr[3])>>8);
	 }
       }
      else
       {
	*dstPtr++ = checkPoint;
	*dstPtr++ = checkPoint;
	*dstPtr++ = checkPoint;
       }
      srcPtr+=4;
     }
   else
    for (x=0; x<wid; x++)
     {
      if (srcPtr[3])
       {
	if (srcPtr[3]==255)
	 {
	  *dstPtr++ = srcPtr[0];
	  *dstPtr++ = srcPtr[1];
	  *dstPtr++ = srcPtr[2];
	 }
	else
	 {
	  invertAlpha = ((int)255-srcPtr[3])<<8;
	  *dstPtr++ = (unsigned char)((invertAlpha + (int)srcPtr[0]*srcPtr[3])>>8);
	  *dstPtr++ = (unsigned char)((invertAlpha + (int)srcPtr[1]*srcPtr[3])>>8);
	  *dstPtr++ = (unsigned char)((invertAlpha + (int)srcPtr[2]*srcPtr[3])>>8);
	 }
       }
      else
       {
	*dstPtr++ = 255;
	*dstPtr++ = 255;
	*dstPtr++ = 255;
       }
      srcPtr+=4;
     }
  }
 /* should be OK to realloc this way, since the memory will always shrink */
 *bpl = wid*3;
 data = realloc(data, hei * *bpl);
 return(data);
}

/* examines the line looking for <tag>value</tag> and return pointer to value, or NULL if not found; will modify line */
DLUTIL char *parseOneLineTagXML(char *line, char *tag)
{
 char *startPos, *endPos;
 char bracketTag[strlen(tag)+5];

 sprintf(bracketTag, "<%s>", tag);
 startPos = strstr(line, bracketTag);
 if (!startPos)
  return(NULL);
 startPos += strlen(bracketTag);
 /* now plant null into the string to null terminate it */
 sprintf(bracketTag, "</%s>", tag);
 /* start searching from the value, not from start of line */
 endPos = strstr(startPos, bracketTag);
 if (endPos)
  endPos[0]=0;
 else
  return(NULL);
 return(startPos);
}
