/***************************************************************************
                          apaintdoc.h  -  description
                             -------------------
    begin                : Mon Jan 27 11:15:43 CET 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef APAINTDOC_H                                      
#define APAINTDOC_H

// include files for QT
#include <qobject.h>
#include <qpixmap.h>
#include <qapp.h>
#include <qwmatrix.h>
#include <qimage.h>
#include <qdragobject.h>
#include "apaintimage.h"
#include <qfiledialog.h>
#include "apaintBoundary.h"

// application specific includes

/**
  * the Document Class
  */

#define SAVE_PATH "./"  
#define MAX_LAYERS  20
#define APAINT_MIME_SIG  "application/apaintLayer"

#define NW  0
#define NE  1
#define SE  2
#define SW  3
#define N 4
#define E 5
#define S 6
#define W 7
#define RECT_SIZE   4

class QMessageBox;
class ApaintBrushList;

enum SelectState{DrawState, MoveState};
enum MousePos{InArea, OutArea, LBorder, RBorder, TBorder, BBorder};

enum LayerChangeType{CreateLayer, ChangeLayerPos, ChangeLayerForm,
 ChangeLayerShiftX, ChangeLayerShiftY, DestroyLayer,
 ChangeLayerMirrorX, ChangeLayerMirrorY, ChangeLayerFlip, ChangeLayerExplode,
 ChangeSelToMainLayer, ChangeMainToSelLayer, ChangeColorTable, Unselect,
 InvertAllColors, InvertRed, InvertGreen, InvertBlue, InvertCyan, InvertYellow, InvertMagenta,
 SwapRGB_GBR, SwapRGB_BRG, SwapRG, SwapRB, SwapGB, SwapColors};
enum LayerType{MainLayer, SelectionLayer, ImageLayer};
enum LayerGeometry{RectangleG, CircleG, Freehand, FreehandSegment};

class Tile {
  
public:
  Tile(QRect rMain, QRect rSecond): posMain(rMain), posSecond(rSecond), mirrorHorOn(false), mirrorVertOn(false), secondHor(false) {};
  ~Tile(){};
  bool mirrorHor(){return mirrorHorOn;}
  bool mirrorVert() {return mirrorVertOn;}
  void setMirrorHor(bool on) {mirrorHorOn = on;}
  void setMirrorVert(bool on) {mirrorVertOn = on;}
  bool intersectsMain(QRect r){ return posMain.intersects(r);}
  bool intersectsSecond(QRect r){ return posSecond.intersects(r);}
  QRect intersectMain(QRect r){ return posMain.intersect(r);}
  QRect intersectSecond(QRect r){ return posSecond.intersect(r);}
  QRect getPosMain() {return posMain;}
  QRect getPosSecond() {return posSecond;}
  void setSecondHor(bool enable) {secondHor = enable;}
  bool getSecondHor(){return secondHor;}
  void setMainPos(QRect r) { posMain = r;}
  void setSecondPos(QRect r) {posSecond = r;}
private:
  QRect posMain, posSecond;
  bool mirrorHorOn;
  bool mirrorVertOn;
  bool secondHor;
};

class LayerDrag: public QMimeSource
{
 public:
  LayerDrag();
  ~LayerDrag();

  const char *format(int i=0) const;
  bool provides(const char *mimeType) const;
  QByteArray encodedData(const char *typ) const;
  void setAImage(ApaintImage *simg);
  void setMask(QImage& m);
  bool usingMask();
  
 private:
  ApaintImage img;
  QImage mask;
  bool useMask;
}; 

class Layer
{
 public:
  Layer(QPointArray &borderPts, LayerType ltyp, LayerGeometry lg, double ax, double ay);
  Layer(LayerType ltyp, LayerGeometry lg, double ax, double ay);
  Layer(int wid, int hei, int depth, uint fillCol, int numCol, LayerType ltyp, double ax, double ay);
  Layer(const QString &fileName, LayerType ltyp, double ax, double ay);  
  Layer(ApaintImage &img, int x, int y, QImage *imask, LayerType ltyp, double ax, double ay);
  ~Layer();
  void initLayer(LayerType ltyp, LayerGeometry lg, double ax, double ay);
  void setNewData(ApaintImage &img, QImage *imask);
  void setNewData(ApaintImage &img, int x, int y, QImage *imask);
  void setupTileTable(int oldx, int oldy, int newx, int newy);
  void createTiles(int sizex, int sizey, int offx, int offy);
  void enableTiles(bool enable);
  bool isUsingTiles() {return useTiles;}
  ApaintImage getTileData(Tile *tdata, QRect r, int sx, int sy, int imgx, int imgy,  bool doMirror);
  ApaintImage getAllTileData(QRect r, bool doMirror);
  void getTileInfo(int &sizex, int &sizey, int &offx, int &offy);
  void setTileList(QPtrList<Tile>& list) { tileList = list;}
  QPtrList<Tile>& getTileList() {return tileList;}
  Tile* pointInTile(int x, int y);
  int getTileNum(int x, int y);
  QPtrList<QRect>& getLayerRefreshArea(QRect area);
  void copyRepeatOptions(Layer *newLayer);
  void updateSize();
  void newPos(int x, int y);
  void move(int dx, int dy);
  bool isValid();
  void loadDataFromLayer(Layer *from, QRect srcPos, bool fillBackground = false);
  bool isDataLoaded();
  void setMask(QImage &smask);
  bool usingMask(){return useMask;}
  void enableMask(bool enable) { useMask = enable;}
  QImage &getMask(){return mask;}
  void xForm(QWMatrix &mx, int dstWid, int dstHei);
  void scaleRotate(QWMatrix &mx, double angle, int dstWidIn, int dstHeiIn, bool resizeContents, uint col,
                   bool rotateContour, bool doAverage, ResizeFilterType ftyp);
  void beginTransform(QPointArray dpts);
  void endTransform();
  QRect layerPos(){return pos;}
  QRect drawAreaPos();
  QRect realLayerPos() {return QRect(pos.x(), pos.y(), imageBuf.width(), imageBuf.height());}
  QRect layerRefreshPos();
  void drawBorderMain(QPainter *p, double zoomX, double zoomY, bool activeLayer);
  void drawBorder(QPainter *p, double zoomX, double zoomY);
  void drawBorder(QPainter *p, QPointArray dpt, double zoomX, double zoomY);
  void setValidDraw(bool isValid);
  LayerType getType(){return type;}
  int getShiftX(){return shiftx;}
  int getShiftY(){return shifty;}
  int getSwapColor1(){return swapColor1;}
  int getSwapColor2(){return swapColor2;}
  void setMask();
  void horShift(int x);
  void vertShift(int y);
  void setSwapColor1(int index) {swapColor1 = index;}
  void setSwapColor2(int index) {swapColor2 = index;}
  void setNumColors(int num, bool preview);
  void removeUnusedColors();
  void setType(int imgType, int numColors, int noise, bool dither, bool preview);
  int  getImageType() { return imageBuf.type();}
  void setColorTable(QRgb *ctab, int num, bool dither, bool preview);
  void setExactColorTable(QRgb *ctab, int num);
  QRgb *getColorTable() {return imageBuf.colorTable();}
  int numColors() {return imageBuf.numColors();}
  void undoPreview();
  void finishPreview();
  LayerDrag *getClipboardData();
  QPixmap *getPixmap() {return &pix1Bit;}
  void setLayerType(LayerType t);
  void setColor(int n, QRgb col);
  bool initPixmap(int wid, int hei);
  void setAspectRatio(double ax, double ay);
  void getUsedColors(bool *uc, int &num);
  void remapData(RemapArray &rmap, bool preview);
  bool isImageNull() {return imageBuf.isNull();}
  LayerGeometry getGeometry() {return geometry;}
  void setGeometry( LayerGeometry g);
  QPointArray *getBorderPoints(){ return &freehandBorder;}
  void setBorderPoints(QPointArray &pts);
  int numCtrlPoints() {return freehandBorder.count();}
  static bool pointVisible(QImage &mask, int xin, int yin);
  bool pointVisible(int xin, int yin);

  void applyFilter(int what, int fgCol, int bgCol, int penX, int penY, int value, bool doVarColors, bool doVarSize, bool doGradient);
  void applyFilter(int x, int y, int w, int h, int what, int fgCol, int bgCol, int penX, int penY, int value, bool doVarColors, bool doVarSize, bool doGradient);

  void contourAverage(int thickX, int thickY);
  void contourAverage(int x, int y, int w, int h, int rad, double aspectX, double aspectY, int thickX, int thickY);
  void contourDespecle(int what, int neighbours);
  void contourDespecle(int x, int y, int w, int h, int what, int neighbours);
  void contourThick(uint col, int direction, int timesX, int timesY, int angle, bool doShade, bool doLines,
  	bool doGradient, bool doBidirectional, bool repeatOperation);
  void contourRemove(uint col);
  void contourThin(uint col);
  void contourAdd(uint fgCol, uint bgCol, uint withCol, bool useWithCol, int direction, bool into, bool doShade, int numX, int numY);
  void contourPrune(uint col);
  void changeToBW(int treshHold, bool preview);
  void mirrorHorizontal();
  void mirrorVertical();
  void mirrorFlip();
  void mirrorExplode();
  void duplicate(int duplicateType);
  void scale2X();
  void scale3X();
  void shrink2X();
  void shrink3X();
  void shrink4X();
  void halve(int halveType);
  void shrink(int shrinkType);
  void multiplyX(int times);
  void multiplyY(int times);
  void invertColors(int what, bool *protColors);
  void swapColors(int first, int second);
  void contrastStretch();
  void makeColorShades(int fgIndex, int bgIndex);
  void sortColors(int sortOrder);
  void perspectiveXForm(Layer *layer, int dstx, int dsty, int dstw, int dsth, double mx[3][3]);
  void borderMirror(bool vert);
  void insertRemovePixelsHor(int x, int num, bool doInsert, bool useBgCol, bool useDataBuf, uchar *dataBuf);
  void insertRemovePixelsVert(int y, int num, bool doInsert, bool useBgCol, bool useDataBuf, uchar *dataBuf);
  bool doDrawDelimiterLines() {return drawDelimiterLines;}
  void setDrawDelimiterLines(bool enable) {drawDelimiterLines = enable;}
  void drawTileLines(QPainter *p, QRect clip, double zoomX, double zoomY);
  bool isTransColor(int i) {return transColors[i];}
  bool isProtectedColor(int i) {return protectedColors[i];}
  void setTransColor(int i, bool enable){ transColors[i] = enable; }
  void setTransColors(bool *transTab);
  void setProtectedColor(int i, bool enable) {protectedColors[i] = enable;}
  void setProtectedColors(bool *protectedTab);
  bool *getTransColors() {return transColors;}
  bool *getProtectColors() {return protectedColors;}
  void resetColors();
  QString getLayerName();
  void setLayerName(QString lname);
  void setLayerName();
  bool isHidden() {return hidden;}
  void show() {hidden = false;}
  void hide() {hidden = true;}
  void tilesToImage();
  QString getBaseName(QString name);
  void undoDataPreview();
  void mergeColors();
  void setResizeable(bool enable) { isResizeable = enable;}
  bool getIsResizeable() {return isResizeable;}
  void setFgIndx(int fg){ fgNum = fg;}
  void setBgIndx(int bg) {bgNum = bg;}
  void setFgCol(QRgb col) {fgCol = col;}
  void setBgCol(QRgb col) {bgCol = col;}
  int getFgIndx() { return fgNum;}
  int getBgIndx() { return bgNum;}
  QRgb getFgCol() { return fgCol;}
  QRgb getBgCol() { return bgCol;}
  void fill(int xr, int yr, int w, int h, uint col, bool useMask, QImage *mask);
  void setMaskWithColor(uint col);
  bool isBorderColor(uint col);
  void setTransperantBg();
  bool isPreviewMode(){ return previewMode;}
  void setPreviewMode(bool enable) {previewMode = enable;}
  void setFilename(QString fname) {filename = fname;}
  QString getFilename() {return filename;}
  void setExactNumColors(int num);
  int indexLastUsedColor() {return imageBuf.indexLastUsedColor();}
  int *getPixSrcZoomX() {return srcZoomX;}
  int *getPixSrcZoomY() {return srcZoomY;}
  void showBorder(bool enable) {doShowBorder = enable;}

    
  ApaintImage imageBuf;
  ApaintImage origBuf;
  
 private: 
  enum ViewMode{TransformView, NormalView};

  void flipBorder();
  void updateLayerData();
  void drawTileLine(QPainter *p, int x1, int y1, int x2, int y2, QRect &clip, bool horizontal);
  void transformBoundary(QWMatrix &mx);
  
  QRect transPos, pos;  
  bool validDraw;/*validDraw: if true the border is drawn, else not*/
  bool valid; /*layer is valid...*/
  bool dataLoaded, useMask;
  QPointArray pts, tpts;
  BoundSeg *ptsBoundary, *transPtsBoundary;
  int numBoundSeg;
  QImage mask, saveMask;
  QPixmap pix1Bit;
  ViewMode vmode;
  LayerType type;
  int shiftx, shifty;
  int swapColor1, swapColor2;
  LayerGeometry geometry;
  QPointArray freehandBorder, xBorder/*for transformations*/;
  int tilex, tiley;
  int tileOffx, tileOffy;
  bool useTiles;
  QPtrList<Tile> tileList;
  QPtrList<QRect> refArea;
  bool drawDelimiterLines;
  bool transColors[256];
  bool protectedColors[256];
  bool hidden;
  bool isResizeable;
  bool previewMode;
  bool doShowBorder;
  QString name, filename;
  int fgNum, bgNum;
  QRgb fgCol, bgCol;
  double aspectX, aspectY;
  int *srcZoomX, *srcZoomY;
};


class ApaintDoc : public QObject
{
  Q_OBJECT

  public:
    ApaintDoc(QWidget *pparent);
    ~ApaintDoc();

    void sendSignals(bool layerChg, bool mainLayerChg);
    void newDoc(int wid, int hei, int depth, uint fillCol, int numCol);
    bool save(bool saveAs, QString &saveFileName);
    void saveLayer();
    bool load(const QString &filename);
    bool loadLayer(const QString &filename);
    bool newLayer(int wid, int hei, int depth, uint fillCol, int numCol);
    bool isModified() const;
    int numLayers() const {return nLayers;}
    
    void createNewLayer(int x, int y, int wid, int hei, LayerType ltyp, LayerGeometry lg, bool fillBackground = false);
    void createNewLayer(QPointArray &borderPts, LayerType ltyp, LayerGeometry lg);
    void createNewLayer(ApaintImage &img, QImage *imask, int x, int y);
    void createNewLayer(QImage *imask, QRect area);
    void createNewLayer(Layer *srcLayer, QImage *imask, QRect area, bool useSrcPos);
    void finishLayerCreate();
    void resizeLayer(int x, int y, int wid, int hei);
    void collapsLayers();
    void collapsLayer(int i, bool cmapAdd);
    void collapsSelectedLayer(bool cmapAdd);
    void layerSetNewData(ApaintImage &img, QImage *imask);
    void layerSetNewData(ApaintImage &img, int x, int y, QImage *imask);
    void beginLayerTransform(QPointArray pt);
    void transformLayer(QWMatrix &mx, int dstWid, int dstHei);
    void transformLayer(QWMatrix &mx, double angle, int dstWid, int dstHei, bool resizeContents,
                        bool rotateContour, bool doAverage, ResizeFilterType ftyp);
    void endLayerTransform();
    bool layerInBox(int i, QRect box);
    bool layerInBox();
    void freeLayers();
    bool isLayerDataLoaded(){return(layerList[selectedLayer]->isDataLoaded());}
    QRect getLayerPos() {return(layerList[selectedLayer]->layerPos());}
    QRect getLayerPos(int i) {return(layerList[i]->layerPos());}
    QRect getLayerRefreshPos() {return layerList[selectedLayer]->layerRefreshPos();}
    QRect getLayerRefreshPos(int i) {return layerList[i]->layerRefreshPos();}
    void  setLayerPos(int x, int y);
    void setLayerType(LayerType typ);
    bool isLayerValid(Layer *layer);
    
    void moveLayer(int dx, int dy);
    Layer *getLayer() {return(layerList[selectedLayer]);}
    Layer *getLayer(int num);
    Layer *getMainLayer() {return(mainLayer);}
    void setLayer(int pos, Layer *layer);
    int getLayerNum() {return selectedLayer;}
    void setLayerNum(int selLayer);
    void setLayerValidDraw(bool valid);
//    void setLayerMask(QImage mask);
//    void setLayerMask();
    LayerType getLayerType();
    bool removeNullLayer();
    void deleteLayer(int layer);
    void deleteLayer(){ deleteLayer(selectedLayer);}
    bool inTransformMode(){ return layerIsTransforming;}    
    void putUndoStack(LayerChangeType ctyp);
    void putUndoStack(Layer *sel, LayerChangeType ctyp);
    void setLayerNumColors(int num, int noise, bool dither, bool preview);
    void setLayerExactNumColors(int num);
    void setLayerImageType(int imgType, bool dither);
    void setLayerColorTable(QRgb *ctab, int num, bool dither, bool preview);
//    void setLayerExactColorTable(QRgb *ctab, int num);
    void undoLayerPreview(bool dataPreview, bool emitRefresh);
    void finishLayerPreview();
    void cutSelectedLayer();
    void copySelectedLayer();
    void pasteSelectedLayer(int posx, int posy);
    void selectionToMainLayer();
    void mainLayerToSelection(Layer *newMainLayer);
    QRgb *getLayerColorTable();
    int layerNumColors();
//    void getLayerUsedColors(bool *uc, int &num);
    void layerRemapData(RemapArray &rmap, bool preview);
    void setLayerBorderPoints(QPointArray &pts);
    bool isLayerPointVisible(int x, int y);

    void layerApplyFilter(int what, int fgCol, int bgCol, int penX, int penY, int value, bool doVarColors, bool doVarSize, bool doGradient);

    void layerContourAverage(int thickX, int thickY);
    void layerContourDespecle(int what, int neighbours);
    void layerContourThick(uint col, int direction, int timesX, int timesY, int angle, bool doShade, bool doLines,
  	bool doGradient, bool doBidirectional, bool repeatOperation);
    void layerContourRemove(uint col);
    void layerContourPrune(uint col);
    void layerContourThin(uint col);
    void layerContourAdd(uint fgCol, uint bgCol, uint withCol, bool useWithCol, int direction, bool into, bool doShade, int numX, int numY);
    void refreshLayer();
    void refreshLayer(QRect pos);
    void refreshLayer(QRect oldPos, QRect newPos, int border);
    void setDirName(QString &name) { dirName = name;}
    void setFileDlg(QFileDialog *dlg) {fileDlg = dlg;}
    void changeLayerToBW(int treshHold, bool preview);
    void layerMirrorHorizontal();
    void layerMirrorVertical();
    void layerMirrorFlip();
    void layerMirrorExplode();
    void layerDuplicate(int duplicateType);
    void layerScale2X();
    void layerScale3X();
    void layerScale4X();
    void layerShrink2X();
    void layerShrink3X();
    void layerShrink4X();
    void layerHalve(int halveType);
    void layerShrink(int shrinkType);
    void layerMultiplyX(int times);
    void layerMultiplyY(int times);
    void layerSortColors(int sortOrder);
    void layerInvertColors(int what);
    void layerSwapColors(int first, int second);
    void layerContrastStretch();
    void layerMakeColorShades(int fgIndex, int bgIndex);
    void layerInvertProtectedColors();
    void layerClearProtectedColors();
    void layerInvertTransparentColors();
    void layerClearTransparentColors();
    void layerPerspectiveXForm(int dstx, int dsty, int dstw, int dsth, double mx[3][3]);
    LayerDrag *getClipboardData();
    void createMainLayerTiles(int sizex, int sizey, int offx, int offy, bool justRefresh);
    void enableMainLayerTiles(bool enable);
    void copyImageToLayer(Layer *layer, QRect area, ApaintImage& img, ApaintImage& origImg, bool sameImage, bool useMask, QImage *mask, bool *transCol, bool cmapAdd);
    void setLayerTransColor(int i, bool enable);
    void setLayerTransColors(bool *transTab);
    void setLayerProtectColor(int i, bool enable);
    void setLayerProtectColors(bool *protectTab);
    void setActiveLayer(int num);
    void newImageFromTiles();
    bool layerPosUp();
    bool layerPosDown();
    void removeLayerUnusedColors();
    void mergeLayerColors();
    void setLayerResizeable(bool enable);
    bool getLayerResizeable();
    void protectLayer(bool enable);
    bool isProtectMode(){return protectMode;}
    void reduceSelectionBorder();
    ApaintBrushList *getBrushList();
    double getAspectX() {return aspectX;}
    double getAspectY() {return aspectY;}
    double getXYratio();
//    double getXYratio() {return (double)layerList[selectedLayer]->imageBuf.xdpi()/(double)layerList[selectedLayer]->imageBuf.ydpi(); }

    
    Layer *layerList[MAX_LAYERS];
    Layer *mainLayer;

  public slots:
    void setBgIndx(int indx);
    void setBgCol(QRgb rgb);
    void setFgIndx(int indx);
    void setFgCol(QRgb rgb);
    void redrawLayers();
    void setAspectRatio(double ax, double ay);

    
  signals:
    void layerChanged(Layer *sel);
    void newDocSig();
    void mainLayerChanged();
    void refreshSig(int x, int y, int w, int h);
    void refreshLayerSig(const QRect& oldRect, const QRect& newRect, int border);
    void putUndoStackSig(Layer *sel, LayerChangeType ctyp);
    void layerPosChanged();
    void layerMovedSig(QRect pos);
    void newLayerSig(Layer *layer);
    void removeLayer(int num);
    void protectLayerSig(bool enable);
    void transProtColorsChanged();
    void layerSelected(int);
    
  private:
    bool fixFilename(QString &fname, int imgType);
                  
    bool modified;
    bool protectMode;
    int selectedLayer, nLayers;
    QWidget *parent;
    bool useClip;
    bool layerIsTransforming;
    QString dirName;
    QFileDialog *fileDlg;
    double aspectX, aspectY;

};

#endif
