/***************************************************************************
                          drawinrepeat.h  -  description
                             -------------------
    begin                : Thu Sep 25 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DRAWINREPEAT_H
#define DRAWINREPEAT_H

#include <qrect.h>
#include <qpainter.h>
#include <qptrlist.h>
#include <qvaluelist.h>

class ApaintDraw;
class QPointArray;

class DrawOp {
public:
  DrawOp():mirrorx(false), mirrory(false), scalex(1), scaley(1){};
  virtual ~DrawOp(){};
  void setArea(QRect a){ area = a;}
  void setAspectArea(QRect a) {aspectArea = a;}
  void setDrawArea(QRect a) {drawArea = a;}
  void setScale(double sx, double sy){ scalex = sx; scaley = sy;};
  void setMirror(bool mx, bool my){mirrorx = mx; mirrory = my;}
  QRect getArea() { return area;}
  QRect getAspectArea() {return aspectArea;}
  QRect getDrawArea() {return drawArea;}
  void mirrorPoint(int x, int y, int *xo1, int *yo1);
//  void mirrorPoint(int x, int y, int &xo1, int &yo1, QRect mRect);
  void mirrorPoint(int x, int y, int *xo1, int *yo1, QRect mRect);
  bool lineInRect(int x1, int y1, int x2, int y2, QRect clip);
  virtual void draw(QPainter *p);
  virtual void refresh(QRect nrect, QRect drawRect, bool useFg, bool final);
  virtual void draw(QPainter *p, QRect imgRect, QRect mrect, QRect srect, bool repeatHor);
  
protected:

 QRect area;
 QRect aspectArea;
 QRect drawArea;
 bool mirrorx, mirrory;
 double scalex, scaley;
};

class EllipseOp: public DrawOp{
public:  
  EllipseOp(ApaintDraw *drawp):DrawOp(), d(drawp) {};
  ~EllipseOp(){};
  void draw(QPainter *p);
  void refresh(QRect nrect, QRect drawRect, bool useFg, bool final);
protected:
  ApaintDraw *d;
};

class RectOp: public DrawOp{
public:
  RectOp(ApaintDraw *drawp):DrawOp(), d(drawp) {};
  ~RectOp(){};
  void draw(QPainter *p);
  void refresh(QRect nrect, QRect drawRect, bool useFg, bool final);
protected:
  ApaintDraw *d;
};

class EllipArcOp: public DrawOp{
protected:
  bool isClosed;
  bool isPreview;
public:
  EllipArcOp(ApaintDraw *drawp):DrawOp(), d(drawp) { isClosed = false; isPreview = false; }
  ~EllipArcOp(){};
  void setDrawPts(QPointArray &pts, int nPts);
  void draw(QPainter *p);
  void refresh(QRect nrect, QRect drawRect, bool useFg, bool final);
  bool preview() const { return isPreview; }
  void setPreview(bool value) { isPreview = value; }
  bool closed() const { return isClosed; }
  void setClosed(bool value) { isClosed = value; }
protected:
  ApaintDraw *d;
  QPointArray dpts;
  int numPts;
};

class LineOp: public DrawOp{
public:
  LineOp(ApaintDraw *drawp):DrawOp(), d(drawp) {};
  ~LineOp(){};
  QRect getLineArea(int xi1, int yi1, int xi2, int yi2);
  void setDrawLine(int xi1, int yi1, int xi2, int yi2);
  void draw(QPainter *p);
  virtual void refresh(QRect nrect, QRect drawRect, bool useFg, bool final);
protected:
 int x1, y1, x2, y2;
 ApaintDraw *d; 
};

class BezierOp: public DrawOp{
public:
  BezierOp(ApaintDraw *drawp):DrawOp(), d(drawp) {};
  ~BezierOp(){};
  void setDrawLine(const QPointArray &points, const QValueList<int> &segments);
  void draw(QPainter *p);
  virtual void refresh(QRect nrect, QRect drawRect, bool useFg, bool final);
protected:
 const QPointArray *points;
 const QValueList<int> *segments;
 ApaintDraw *d;
};

class BrushOp: public LineOp{
public:
  BrushOp(ApaintDraw *drawp):LineOp(drawp){};
  ~BrushOp(){};
  void refresh(QRect nrect, QRect drawRect, bool useFg, bool final);
};

class ScreenLineOp: public DrawOp{
public:
  ScreenLineOp():DrawOp() {};
  ~ScreenLineOp(){};
  void setLineArea(int xi1, int yi1, int xi2, int yi2);
  virtual void draw(QPainter *p, QRect imgRect, QRect mrect, QRect srect, bool repeatHor);
private:
  int x1, y1, x2, y2;
};

class ApaintDoc;
class ApaintDraw;

class DrawInRepeat {
public: 
	 DrawInRepeat(ApaintDoc *docp, ApaintDraw *drawp);
	~DrawInRepeat(); 

  void drawRect(QPainter *p, QRect rect, bool doRefresh, bool useFg, bool final);
  void drawRectWithoutAspect(QPainter *p, QRect rect, bool doRefresh, bool useFg, bool final);
  void drawEllipse(QPainter *p, QRect ellipse, bool doRefresh, bool useFg, bool final);
  void drawLine(QPainter *p, int xr1, int yr1, int xr2, int yr2, bool doRefresh, bool useFg, bool final);
  void drawBezier(QPainter *p,
		  const QPointArray &oldPoints, const QPointArray &points,
		  const QValueList<int> &erasingSegments, QValueList<int> &drawingSegments,
		  int oldlineWidth, int lineWidth,
		  bool final);
  void drawScreenLine(QPainter *p, int x1, int y1, int x2, int y2, double scalex, double scaley);
  void drawBrush(QPainter *p, int xr1, int yr1, int xr2, int yr2, bool doRefresh, bool useFg, bool final);
  void drawPolygon(QPainter *p, QPointArray &rpts, bool doRefresh, bool useFg, bool final);
  void drawClosedPolygon(QPainter *p, QPointArray &rpts, int nRPts, bool doRefresh, bool useFg, bool final);
  void drawPolygon(QPainter *p, QPointArray &rpts, int nRPts, bool doRefresh, bool useFg, bool final);
  void transPoint(int xin, int yin, int *xout, int *yout);
  void refreshDirect(int x, int y, int w, int h, bool useFg, bool doDraw, bool final);
  void refreshImageDraw(int x, int y, int w, int h, bool final);
  void refreshIntersectedArea(const QRect &oldRec, const QRect &newRec, int border);
  void refreshArea(int x1, int y1, int x3, int y2, int border);
  void clearRect(int lineWid, int xr1, int yr1, int xr2, int yr2);
  
protected:

  void drawIt(QPainter *p, DrawOp *op, bool doRefresh, bool useFg, bool final);
  void drawItOnScreen(QPainter *p, DrawOp *op);
  
private:

  void adjustWithPenSize(QPainter *p, QRect &area, bool isDrawArea);
  void adjustPolygon(QPointArray &ptsIn, QPointArray &ptsOut, double ax, double ay);
  QRect getLineArea(int xi1, int yi1, int xi2, int yi2);
  void drawLineBase(DrawOp *op, QPainter *p, int xr1, int yr1, int xr2, int yr2, bool doRefresh, bool useFg, bool final);
  void drawRectBase(DrawOp *op, QPainter *p, QRect rect, bool doRefresh, bool useFg, bool final);
  
  ApaintDoc *doc;
  ApaintDraw *draw;
  EllipseOp *ellipOp;
  LineOp *lineOp;
  RectOp *rectOp;
  BrushOp *brushOp;
  EllipArcOp *eArcOp;
  ScreenLineOp *srcLineOp;
  BezierOp *bezierOp;
};

#endif
