/***************************************************************************
                          apaintview.h  -  description
                             -------------------
    begin                : Mon Jan 27 11:15:43 CET 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef APAINTVIEW_H
#define APAINTVIEW_H

// include files for QT
#include <qwidget.h>
#include <qscrollview.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qcursor.h>
#include <stdio.h>
#include <math.h>
#include <qimage.h>
#include <qbitmap.h>
#include <qspinbox.h>
#include "dialog_ui_gridProp.h"

// application specific includes
#include "apaintdoc.h"

extern bool autoScrollNoButton;

#define MAX_SCALE 30
#define MIN_SCALE 0.05
/**
 * This class provides an incomplete base for your application view. 
 */
class ApaintIO;
class ApaintBrushList;

class ApaintView : public QScrollView
{
  Q_OBJECT
  public:
    ApaintView(QWidget *parent, ApaintDoc *docp);
    ApaintView(QWidget *parent, const char *name);
    ~ApaintView();

    void setIntegerView(bool enable);
    void setAntialiasView(bool enable);
    bool antialias() const;
    QRect getContentsSize();
    void setImage(ApaintImage *imgp);
    double getScaleX() {return zoomX;}
    double getScaleY() {return zoomY;}    
    double getViewScaleX();
    double getViewScaleY();

    //Guide change
    int getOffsetX() {return offsetX;}
    int getOffsetY() {return offsetY;}
    //Guide change end
         
  signals:
    void mouseMoveSig(QMouseEvent *e, QPainter *pc);
    void mousePressSig(QMouseEvent *e, QPainter *pc);
    void mouseReleaseSig(QMouseEvent *e, QPainter *pc);
    void mouseDoubleClickSig(QMouseEvent *e, QPainter *pc);
    void keyPressSig(QKeyEvent *e, QPainter *pc);
    void keyReleaseSig(QKeyEvent *e, QPainter *pc);
    void mouseWheelSig(QWheelEvent *e);
    void leaveEventSig(QEvent *e, QPainter *pc);
    void enterEventSig(QEvent *e, QPainter *pc);
    void initView();
    void zoomChanged(double zx, double zy);
    void viewZoomChanged(double zx, double zy);
    void contentsSizeChanged(int w, int h);
    void docPropChanged(double zx, double zy);
    void drawContentsSig(QPainter *p, int clipx, int clipy, int clipw, int cliph);    
    void viewOffsetChanged(int x, int y);
    void beforeRefresh(QPainter *pc);
    void afterRefresh(QPainter *pc);
    void scaleFactorChanged();
    void repaintSig(QPainter *pc);

    //Guide change
    void repaintGuideSig(); 
    //Guide change end

  public slots:
    void scale(double x, double y, bool center, int posx, int posy);
    void scaleUp(int posx, int posy, bool center);
    void scaleDown(int posx, int posy, bool center);
    void setViewCur(QCursor &cur);
    void unsetViewCur();
    void refresh(int x, int y, int wid, int hei);
    void setAutoScroll(bool On);
    void documentChangedSlot();
    void showGrid(bool show);
    void scaleFactorSlot(double fx, double fy);
    void setGridColor(uchar col);
    void setGridSize(int w, int h);
    void setVisible(QRect pos);
    void refreshSelColor();
    void highlightSelColor();
    void repaint();

    //Guide change
    void repaintGuideSlot();
    //Guide change end

  protected slots:
    void contentsMovingSlot(int x, int y);

  protected:
    virtual void keyPressEvent( QKeyEvent *e );
    virtual void keyReleaseEvent( QKeyEvent *e);
    virtual void drawContents(QPainter *p, int clipx, int clipy, int clipw, int cliph);
    virtual void contentsMousePressEvent (QMouseEvent *e );
    virtual void contentsMouseReleaseEvent (QMouseEvent *e );
    virtual void contentsMouseDoubleClickEvent (QMouseEvent *e );
    virtual void contentsMouseMoveEvent (QMouseEvent *e );
    virtual void contentsWheelEvent(QWheelEvent *e);
    virtual void leaveEvent(QEvent *e);
    virtual void enterEvent(QEvent *e);
    
    //virtual void viewportResizeEvent(QResizeEvent *e);

    void setZoom(double zoomValX, double zoomValY, bool center, int posx, int posy);
    void setImagePos(int x, int y, double oldZoomX, double oldZoomY);
    void setCenterImagePos(double oldZoomX, double oldZoomY);
    void setCanvasPainter(QPainter *p);

 protected:
    enum ViewType {DocumentView, ImageView};
    void init(ViewType type);
    void drawContentsDoc(QPainter *p, int clipx, int clipy, int clipw, int cliph);
    void drawContentsImg(QPainter *p, int clipx, int clipy, int clipw, int cliph);
 private:
    void scrollCorner(int x, int y);
    
    bool documentLoaded, repaintFromMoving, firstMove;
    int sizeX,sizeY;
    int dstx,dsty,dstx2,dsty2, dstw, dsth;
    int zoomSizeX,zoomSizeY;
    int clipWidth,clipHeight;
    int mouseX, mouseY;
    bool dirty;
    bool autoScroll, lbtnPressed, rbtnPressed, gridOn;
    double zoomX, zoomY, facX, facY, oldFacX, oldFacY, mzoomX, mzoomY;    
    ApaintIO *io;
    ApaintDoc *doc;
    ApaintImage *img;
    bool swRepeat;
    bool integerView;
    bool antialiasView;
    ViewType vType;
    ApaintBrushList *brushList;
    
    //Guide change
    int offsetX;
    int offsetY;
    //Guide change end
};

class GridPropDlg: public GridPropDialogUI  {
  Q_OBJECT
public:
  GridPropDlg(QWidget *parent, const char *name);
  ~GridPropDlg();

  void showDlg();
  uchar getGridColor() {return gridColor;}
  void setGridSize(int x, int y) {sbWid->setValue(x); sbHei->setValue(y);}
  int getWid() {return sbWid->value();}
  int getHei() {return sbHei->value();}
signals:
  void okBtnClicked();

protected slots:
  void colorClicked(int val);
  void sizeChanged(int val);
  void closeDlg();

private:
  uchar gridColor;
};


#endif
