/***************************************************************************
                          colorpalette.cpp  -  description
                             -------------------
    begin                : Fri Apr 11 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dialogColorPalette.h"
#include <qpen.h>
#include <qbrush.h>
#include <qpainter.h>
#include <qdrawutil.h>
#include <qspinbox.h>
#include <stdlib.h>
#include <qslider.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qtooltip.h>
#include "apaintconfig.h"

extern char *getCommonPNG(const char *pixName);

#define CELL_SIZE_X 27
#define CELL_SIZE_Y 27
#define CELL_OFFSET 1
#define XOFFSET     3
#define YOFFSET     5

#define COLOR_MED           381

/*COLOR PALETTE DILAOG CLASS*/

ColPaletteDlg::ColPaletteDlg( QWidget* parent,  const char* name, ApaintDoc *docp)
    : QWidget( parent, name)
{    
    if ( !name )
      setName( "ColPaletteDlg" );
    resize( 174, 212 );
//    setCaption( trUtf8( "Color palette" ) );
    ColPaletteDlgLayout = new QVBoxLayout( this, 1, 1, "ColPaletteDlgLayout");

    colPWidget = new ColorPaletteImage( this,  "colorP", docp);
    ColPaletteDlgLayout->addWidget( colPWidget );

    Layout1 = new QHBoxLayout( 0, 0, 6, "Layout1");

    btnRefresh = new QPushButton( this, "btnRefresh" );
    btnRefresh->setIconSet(QPixmap(getCommonPNG("dialog-information")));
    Layout1->addWidget( btnRefresh );
    QToolTip::add(btnRefresh, ApaintConfig::getText(649));
    btnRemove = new QPushButton( this, "btnRemove");
    btnRemove->setIconSet(QPixmap(getCommonPNG("edit-delete")));
    Layout1->addWidget(btnRemove);
    QToolTip::add(btnRemove, ApaintConfig::getText(654));
    btnMerge = new QPushButton(this, "btnMerge");
    btnMerge->setIconSet(QPixmap(getCommonPNG("arrow-join")));
    QToolTip::add(btnMerge, ApaintConfig::getText(655));
    Layout1->addWidget(btnMerge);
    btnPlus = new QPushButton(this, "btnPlus");
    btnPlus->setIconSet(QPixmap(getCommonPNG("list-add")));
    QToolTip::add(btnPlus, ApaintConfig::getText(710));
    Layout1->addWidget(btnPlus);
    btnMinus = new QPushButton(this, "btnPlus");
    btnMinus->setIconSet(QPixmap(getCommonPNG("list-remove")));
    QToolTip::add(btnMinus, ApaintConfig::getText(711));
    Layout1->addWidget(btnMinus);
    
    QSpacerItem* spacer = new QSpacerItem( 30, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout1->addItem( spacer );
    ColPaletteDlgLayout->addLayout( Layout1 );
    connect(colPWidget, SIGNAL(colorChanged(QRgb , int)), this, SIGNAL(colorChanged(QRgb , int)));
    connect(colPWidget, SIGNAL(putColorUndo(QRgb, int)), this, SIGNAL(putColorUndo(QRgb, int)));
    connect(colPWidget, SIGNAL(bgFgChanged(int , QRgb , int , QRgb )), this, SIGNAL(bgFgChanged(int , QRgb , int , QRgb )));
    connect(btnRefresh, SIGNAL(clicked()), this, SLOT(showUsedColors()));
    connect(btnRemove, SIGNAL(clicked()), this, SLOT(removeUnusedColors()));
    connect(btnMerge, SIGNAL(clicked()), this, SLOT(mergeColors()));
    connect(btnPlus, SIGNAL(clicked()), this, SLOT(addColor()));
    connect(btnMinus, SIGNAL(clicked()), this, SLOT(removeColor()));
    Q_CHECK_PTR(docp);
    doc = docp;
}

/*
 *  Destroys the object and frees any allocated resources
 */
ColPaletteDlg::~ColPaletteDlg()
{
    // no need to delete child widgets, Qt does it all for us
}

void ColPaletteDlg::mergeColors()
{
  if (doc->isProtectMode()) return;
  doc->putUndoStack(ChangeColorTable);
  colPWidget->mergeSameColors();
}
    
void ColPaletteDlg::setColorPos(int num)
{
  colPWidget->setColorPos(num);
}

void ColPaletteDlg::setColor(QRgb rgb, bool doUndo)
{
  colPWidget->setColor(rgb, doUndo);
}

void ColPaletteDlg::layerChangedSlot(Layer *layer)
{
  colPWidget->layerChangedSlot(layer);
}

void ColPaletteDlg::removeUnusedColors()
{
 if (doc->isProtectMode())
  return;
 doc->putUndoStack(ChangeColorTable);
 doc->removeLayerUnusedColors();
}

void ColPaletteDlg::showUsedColors()
{
  showUsedColors(true);
}

void ColPaletteDlg::showUsedColors(bool enable)
{
  colPWidget->showUsedColors(enable);
}

void ColPaletteDlg::disableUsedColors()
{
  if (colPWidget->isShowingUsedColors())
    colPWidget->showUsedColors(false);
}

QRgb ColPaletteDlg::getColor(int indx)
{
  QRgb *ptab = colPWidget->getColorTab();
  if (!ptab) return 0;
  if (indx<0 || indx>255) return 0;
  return ptab[indx];
}

void ColPaletteDlg::addColor()
{
 if (!doc->getLayer()) return;
 if (doc->isProtectMode()) return;
 if (doc->getLayer()->getImageType() == LOAD_TRUE_COL) return;

 int num = doc->layerNumColors();
 if (num<256)
  {
   int newRGB[3] = {0, 0, 0};
   int bestRGB[3] = {0, 0, 0};
   int i, j, diff, minDiff, bestDiff;
   QRgb *ctab; 

   doc->putUndoStack(ChangeColorTable);
   ctab = doc->getLayer()->imageBuf.colorTable();

   /* create 500 random colors and pick the one which is most different from all the colors in existing color palette */
   bestDiff = 0;
   for (i=0; i<500; i++)
    {
     newRGB[0] = rand()&255;
     newRGB[1] = rand()&255;
     newRGB[2] = rand()&255;
     minDiff=INT_MAX;
     for (j=0; j<num; j++)
      {  /* we make green the most important for color difference, as it contributes most to the lightness */
       diff  = (qRed(ctab[j])-newRGB[0])*(qRed(ctab[j])-newRGB[0])*3;
       diff += (qGreen(ctab[j])-newRGB[1])*(qGreen(ctab[j])-newRGB[1])*4;
       diff += (qBlue(ctab[j])-newRGB[2])*(qBlue(ctab[j])-newRGB[2])*2;
       if (diff<minDiff)
	minDiff = diff;
      }
     if (minDiff>bestDiff)
      {
       bestDiff=minDiff;
       bestRGB[0]=newRGB[0];
       bestRGB[1]=newRGB[1];
       bestRGB[2]=newRGB[2];
      }
     }
   doc->setLayerNumColors(num+1, 0, false, false);
   colPWidget->setColorPos(num);
   colPWidget->setColor(qRgb(bestRGB[0], bestRGB[1], bestRGB[2]), FALSE);
  }
 else
  QApplication::beep();
}

void ColPaletteDlg::removeColor()
{
 Layer *layer = doc->getLayer();
 if (!layer) return;
 if (doc->isProtectMode()) return;
 if (layer->getImageType() == LOAD_TRUE_COL) return;

 if (layer->numColors()-1>layer->indexLastUsedColor())
  {
   doc->putUndoStack(ChangeColorTable);
   doc->setLayerExactNumColors(layer->numColors()-1);
  }
 else
  QApplication::beep();
}

int ColPaletteDlg::getColorPos()
{
  return colPWidget->getColorPos();
}

/************************COLORPALETTE CLASS******************************************/
    
ColorPalette::ColorPalette(QWidget *parent, const char *name):QScrollView(parent, name)
{

 allowColorDelete = false;
 setColorNum(0);
 nc = nr = 1;
 colorTable = 0;
 setValidTable(false);
 for (int i=0; i<256; i++)
  visibleColors[i] = true;  
 viewport()->setFocusPolicy(QWidget::StrongFocus);
 setHScrollBarMode(AlwaysOff);
}

QSize ColorPalette::sizeHint() const
{  
 return QSize(4*CELL_SIZE_X, CELL_SIZE_Y);
}

ColorPalette::~ColorPalette()
{
 if (colorTable)
  delete[] colorTable;
}

void ColorPalette::setVisibleColors(bool *visibleCol)
{
 for (int i=0; i<256; i++)
  visibleColors[i] = visibleCol[i];

 updateContents();
}

void ColorPalette::showColor(int i)
{
 visibleColors[i] = true;
 repaintContents(1);
}

void ColorPalette::hideColor(int i)
{
 visibleColors[i] = false;
 repaintContents(1);
}

void ColorPalette::viewportResizeEvent(QResizeEvent *re)
{
  QScrollView::viewportResizeEvent(re);
  setUpContents(colorNum());
}

void ColorPalette::setUpContents(int num)
{
 int wid, hei;
 int col, row;
  
// nc = width() / (CELL_SIZE_X+CELL_OFFSET);
 nc = max(CELL_SIZE_X, width() - CELL_SIZE_X/2) / CELL_SIZE_X ;
 
 nr= num/nc;
 if (num%nc>0) nr++;
 wid=nc*(CELL_SIZE_X+CELL_OFFSET);
 hei=nr*(CELL_SIZE_Y+CELL_OFFSET);
 resizeContents(wid, hei);
 /*if last color is not in last column than we have to refresh last line*/
 getColRow(num-1, col, row); 
 if (col != nc)
 {
   QRect r1 = getCell(col, row);
   QRect r2 = getCell(nc, nr);
   repaintContents(r1.left(), r1.top(), r2.right(), r2.bottom());
 }
}

QRgb *ColorPalette::getTransColorTab()
{
  for (int i=0, n =0 ; i<colorNum(); i++)
  {
   if (visibleColors[i])
   { 
    transColorTab[n] = colorTable[i];
    n++;
   }
  }
  return transColorTab;
}

int ColorPalette::visibleColorNum()
{
  int n = 0;
  for (int i =0; i<colorNum(); i++)
   if (visibleColors[i]) n++;

  return n;
}

void ColorPalette::initColorTable(int num)
{
 nc=nr=0;
 if (colorNum())
 {
  delete[] colorTable;
  colorTable = 0;
 }
 if (num>0)
 { 
  colorTable = new QRgb[num];
  if (!colorTable)
   num = 0;
 }
 setColorNum(num);
 if (!num)
  setValidTable(false);
}

void ColorPalette::setColorTable(QRgb *tab, int num)
{ 
 initColorTable(num);
 for (int i=0; i<num; i++)
  colorTable[i]=tab[i];
 setUpContents(num);
 setValidTable(true);
}

/*
bool ColorPalette::colorExist(QRgb col)
{
  for (int i=0; i<colorNum(); i++)
   if (colorTable[i] == col)
    return true;
    
  return false;  
}
*/

void ColorPalette::getColRow(int i, int &col, int &row)
{
  int n;
 
  col = row = n =0;  
  for (int j =0; j<colorNum(); j++)
  {
    if (visibleColors[j]) n++;
    if (n == (i+1))
    {
      col = j%numCols();
      row = j/numCols();
      return;
    }    
  }
}

int ColorPalette::getColorPos()
{
  return (getColorPos(selCol, selRow));
}

int ColorPalette::getColorPos(int col, int row)
{
 int n, j, i;
 
 i=row*numCols()+col;
 n =0;
 for (j = 0; j<colorNum(); j++)
 {
  if (visibleColors[j]) n++;
  if (n == (i+1)) return j;
 }
 return 0;
}

void ColorPalette::setColorPos(int indx)
{
  int  x, y;

  if (!isValidTable()) return;

  getColRow(indx, x, y);
  cellClicked(LeftButton, x, y, false);
}

void ColorPalette::setColor(QRgb rgb, bool doUndo)
{
 if (!isValidTable()) return;
 int i=getColorPos(selCol, selRow);
  if (!selectedIsChanged && doUndo)
 {
   selectedIsChanged = true;
   emit putColorUndo(colorTable[i], i);
 } 
 colorTable[i] = rgb;
 emit colorChanged(rgb , i);
 
 refreshCell(selCol, selRow);
}

QRect ColorPalette::getCell(int col, int row)
{
  return QRect(col*(CELL_SIZE_X+CELL_OFFSET), row*(CELL_SIZE_Y+CELL_OFFSET),
               CELL_SIZE_X, CELL_SIZE_Y);
}

void ColorPalette::getCords(int x, int y, int *col, int *row)
{
  *col=x/(CELL_SIZE_X+CELL_OFFSET);
  *row=y/(CELL_SIZE_Y+CELL_OFFSET);
}

void ColorPalette::cellClicked(ButtonState bs, int col, int row, bool doubleClick)
{
  int oldCol, oldRow, i;

  oldCol=selCol;
  oldRow=selRow;
  selCol=col;
  selRow=row;

  i=getColorPos(selCol, selRow);  
  refreshCell(oldCol, oldRow);  
  refreshCell(selCol, selRow);
  emit colorChanged(colorTable[i],  i);
  selectedIsChanged = false;
}

void ColorPalette::refreshCell(int col, int row)
{
  QRect r=getCell(col, row);
  repaintContents(r.x()-1, r.y()-1, r.width()+2, r.height()+2);
}

void ColorPalette::drawProperties(QPainter *p,  QRect r, int n)
{ 
  int light = qRed(colorTable[n])+qGreen(colorTable[n])+qBlue(colorTable[n]);
  if (light<COLOR_MED)
   p->setPen(QPen(white));
  else
   p->setPen(QPen(black));
 
  p->drawText(r.x(), r.y(), r.width(), r.height(), AlignCenter, QString("%1").arg(n + ApaintConfig::getFirstPaletteColor()));
}

void ColorPalette::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cliph)
{
 QRect r, clipRect;
 int x, y, i, num;
 QBrush b;

 if (!isValidTable()) return;
 clipRect = QRect(clipx-1, clipy-1, clipw+2, cliph+2);
  
 if (!colorNum())
  {
    p->eraseRect(clipx,clipy,clipw,cliph);
    return;
  }
 for (i=0, num=0; i<colorNum() ; i++)
  {
    if (!visibleColors[i]) continue;
    x=num%numCols();
    y=num/numCols();
    r = getCell(x, y);
    if (r.intersects(clipRect))
    {
     b=QBrush(colorTable[i]);
     if (x == selCol && y == selRow)
      qDrawShadeRect(p, r, colorGroup(), true, 1, 0, &b);
     else
      qDrawShadeRect(p, r, colorGroup(), false, 1, 0, &b);    
     drawProperties(p, r, i);
    }
    else
     p->eraseRect(r);
    num++;
  }
}

void ColorPalette::contentsMousePressEvent (QMouseEvent *e )
{
 int col, row, i;
 int oldSelCol;

 if (!isValidTable()) return;

  oldSelCol = getColorPos(selCol, selRow); 
  getCords(e->x(), e->y(), &col, &row);
  i=getColorPos(col, row);
  if (col<numCols() && row <numRows() && i<colorNum())
   {
    cellClicked(e->button(), col, row, false);

    if ((e->state() == ControlButton) && (oldSelCol!=i))
     doc->layerSwapColors(i, oldSelCol);
   }
}

void ColorPalette::contentsMouseDoubleClickEvent (QMouseEvent *e )
{
 int col, row;
 int i;

  if (!isValidTable()) return;
  
  getCords(e->x(), e->y(), &col, &row);
  i=getColorPos(col, row);
  if (col < numCols() && row < numRows() && i < colorNum())
   cellClicked(e->button(), col, row, true);
}

void ColorPalette::keyPressEvent(QKeyEvent *e )
{
  if (!isValidTable()) return;
  if (!allowColorDelete) return;
  if (e->key() == Qt::Key_Delete)
  {
    int i = getColorPos(selCol, selRow);
    hideColor(i);
  }
}

QRgb ColorPalette::getSelColor(int &indx)
{
  if (!isValidTable()) return 0;
  int i = getColorPos(selCol, selRow);
  indx = i;
  return (colorTable[i]);
}


ColorPaletteImage::ColorPaletteImage(QWidget *parent, const char *name,  ApaintDoc *docp):ColorPalette(parent, name)
{
  docLoaded=false;
  shUsedCol = false;
  for (int i=0; i<256; i++)
   usedColorsShow[i] = true;
  Q_CHECK_PTR(docp);
  doc = docp;
  ColorPalette::setDoc(docp);
  viewport()->setFocusPolicy(QWidget::NoFocus);
  locked = QPixmap(QPixmap(getCommonPNG("lock-16")));
  transparent = QPixmap(QPixmap(getCommonPNG("transparency-16")));
  enableColorDelete(false);
}

ColorPaletteImage::~ColorPaletteImage()
{

}

QSize ColorPaletteImage::sizeHint() const
{
 return(QSize(5*(CELL_SIZE_X+CELL_OFFSET), 10*(CELL_SIZE_Y+CELL_OFFSET))); 
}   

void ColorPaletteImage::layerChangedSlot(Layer *sel)
{
 int fgn, bgn;
 
 docLoaded=false;
 if (!sel) return;

 selLayer = sel;

 if (selLayer->isImageNull()) return;
 if (selLayer->getImageType()!=LOAD_CMAP)
 {
   /*24 bit ---> init and get out*/
   initColorTable(0);
   updateContents();
   updateGeometry();
   return;
 }
 if (!selLayer->numColors()) return;
 docLoaded=true;
 setColorTable(selLayer->getColorTable(), selLayer->numColors());
 /*selRow and selCol not init but I check i good enough hopefully */
 fgn = selLayer->getFgIndx();
 if (fgn<0) fgn = 1;
 bgn = selLayer->getBgIndx();
 if (bgn<0) bgn = 0;
 fgn%=selLayer->numColors();
 bgn%=selLayer->numColors();
 getColRow(fgn, selCol, selRow);

 emit bgFgChanged(fgn, colorTable[fgn], bgn, colorTable[bgn]);
 if (shUsedCol)
  showUsedColors(true);
 else
  updateContents();
}

void ColorPaletteImage::drawProperties(QPainter *p,  QRect r, int n)
{
  int light = qRed(colorTable[n])+qGreen(colorTable[n])+qBlue(colorTable[n]);
  if (light<COLOR_MED || selLayer->isTransColor(n))
   p->setPen(QPen(white));
  else
   p->setPen(QPen(black));

  if (selLayer->isProtectedColor(n))
   p->drawPixmap(r.x()+5, r.y()+5, locked);
  else
   if (selLayer->isTransColor(n))
    p->drawPixmap(r.x()+5, r.y()+5, transparent);
  QFont fnt = p->font();
  if (shUsedCol && usedColorsShow[n])
    fnt.setUnderline(true);
  else
    fnt.setUnderline(false);
  p->setFont(fnt);
  p->drawText(r.x(), r.y(), r.width(), r.height(), AlignCenter, QString("%1").arg(n + ApaintConfig::getFirstPaletteColor()));
}

void ColorPaletteImage::cellClicked(ButtonState bs, int col, int row, bool doubleClick)
{
 static int goodOldPos=-1;
 int oldPos = getColorPos(selCol, selRow);  
 int i = getColorPos(col, row);   
 
 if (bs == RightButton)
  {
   ColorPalette::cellClicked(bs, col, row, doubleClick);
   QRgb oc = selLayer->imageBuf.color(oldPos);
   setColor(oc, true) ;
  }
 else if (bs == MidButton)
  {   
   QRgb oc = selLayer->imageBuf.color(oldPos);
   QRgb nc = selLayer->imageBuf.color(i);
   setColor(nc, true);
   ColorPalette::cellClicked(bs, col, row, doubleClick);
   setColor(oc, true);
  }
 else if (doubleClick && bs == LeftButton)
  {
   if (selLayer->getType()==MainLayer)
    {
     if (selLayer->isProtectedColor(i))
      doc->setLayerProtectColor(i, false);
     else
      doc->setLayerProtectColor(i, true);
    }
   else
    {
     if (selLayer->isTransColor(i))
      {
       doc->setLayerTransColor(i, false);
       doc->setLayerProtectColor(i, true);
      }
     else if (selLayer->isProtectedColor(i))
      doc->setLayerProtectColor(i, false);
     else
      doc->setLayerTransColor(i, true);
    }
   ColorPalette::cellClicked(bs, col, row, doubleClick);
   /* we don't want that double click on protected/transparent color changes selected color */
   if (goodOldPos>=0)
    setColorPos(goodOldPos);
  }
 else if (bs == LeftButton)
  { 
   goodOldPos = oldPos;
   ColorPalette::cellClicked(bs, col, row, doubleClick);
  }
}

void ColorPaletteImage::setColor(QRgb rgb,  bool doUndo)
{
 
 if (!docLoaded  || !isValidTable()) return;

 int i=getColorPos(selCol, selRow);
 ColorPalette::setColor(rgb, doUndo);
 selLayer->setColor(i, rgb); 
 doc->refreshLayer();

}
void ColorPaletteImage::showUsedColors(bool enable)
{
  if (!docLoaded  || !isValidTable()) return;
  int n;
  
  selLayer->getUsedColors(usedColorsShow, n);
  shUsedCol = enable;
  repaintContents();
}

void ColorPaletteImage::mergeSameColors()
{
  if (!docLoaded  || !isValidTable()) return;

  doc->mergeLayerColors();
}


