/***************************************************************************
;                          colorpalette.h  -  description
                             -------------------
    begin                : Fri Apr 11 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COLORPALETTEUTIL_H
#define COLORPALETTEUTIL_H

#include "dialogColorPalette.h"
#include "dialog_ui_colorNum.h"
#include "dialog_ui_reduceCol.h"
#include "dialog_ui_colorRemap.h"

/*****************************ColorRemap CLASS*********************************************/


class ColorRemap;
  
class ShowColorRemap: public QScrollView {
  Q_OBJECT
public:
  ShowColorRemap(QWidget *parent, const char *name);
  ~ShowColorRemap();
  
  void setColorTable(QRgb *colTabel, int numColors);
  bool isEmpty();
  void getRemapArray(RemapArray &rmap);
  
signals:
  void nodeColorSig(int num);
  void colorRemoved();
          
public slots:
  void insertMainColor(int num);
  void insertNodeColor(int num);
  void removeMainColorPos(int numPos);
  void reset();
  
protected:
  bool colorMapIntersect(int numMap, int clipx, int clipy, int clipw, int cliph, QRect &dstRect);
  QRect getNodeMapSize(int mapNum);
  QRect getColorMapSize(int mapNum);
  void drawColorMap(QPainter *p, int mapNum, QRect &clip);
  void drawContents(QPainter *p, int clipx, int clipy, int clipw, int cliph);
  void contentsMousePressEvent (QMouseEvent *e );
  QSize sizeHint() const;
  void colorMapClicked(int mapNum, int nodeNum, ButtonState bt, QPoint ptClick);
  void drawNumber(QPainter *p,  int x, int y, int w, int h, int n);

private slots:
  void rMenuClicked(int which);  
  
private:
  void updateContentsSize(int mapNum);  

  QRgb colTab[256];
  int numCol;
  ColorRemap *colMap;
  int colMapSel, lastMap;
  int csizex, csizey;
  QPopupMenu *rMenu;
  QPixmap leftArrPix;
};  

/*************************ColorRemapDialog CLASS*******************************/

class ColorRemapDialog: public ColorRemapDialogUI {
  Q_OBJECT
public:
  ColorRemapDialog(QWidget *parent, const char *name, ApaintDoc *docp);
  ~ColorRemapDialog();

  void startDlg();
  void setColorTable(QRgb *colTab, int num);
  bool isOn() {return dlgOn;}

signals:  
  void sendEvents(bool on);
  
public slots:
  void insertMainColor(int num);
  void insertNodeColor(QRgb rgb, int num);
  void layerChangedSlot(Layer *sel);
  void colorFromRemap(int num);
  void okClicked();
  void cancelClicked();
  void previewClicked();

protected slots:
  void doUpdate();
  
protected:
  void closeEvent(QCloseEvent *e);
  void hideEvent(QHideEvent *e);
  void resetData();

private:
  void doUndo(bool refresh);
  void doAction(bool preview);
  void doOnClose();

  ColorPalette *colSelect;
  ShowColorRemap *colRemap;
  QVBoxLayout *vbox1, *vbox2;
  bool dlgOn;
  ApaintDoc *doc;
  bool usedColors[256];
  bool previewOn;
};

                
/***************************ReduceColorDlg*******************************************************/

class QFileDialog;
class IconViewDlg;

class ReduceColorDlg: public ReduceColDialogUI {
 Q_OBJECT
public:
  ReduceColorDlg(QWidget *parent, const char *name, ApaintDoc *docp, QFileDialog *fDlg, IconViewDlg *iDlg);
  ~ReduceColorDlg();
  void startDlg();
  bool isOn() {return isOpen;}

public slots:
  void btnOkClicked();
  void btnPreviewClicked();
  void btnCancelClicked();
  void showColor(int num);
  void layerChangedSlot(Layer *layer);

signals:
  void sendEvents(bool on);

protected slots:
  void setBWBorder(int num);
  void setMode(int showMode);
  void loadBtnPalette();
  void browseBtnPalette();
  void loadPalette(QString &fname);
  void loadPaletteOK(QString &fname);
  void loadPaletteDitherOK(QString &fname);
  void clearBtnPalette();
  void doUpdate();
  
protected:
  void closeEvent(QCloseEvent *e);
  void hideEvent(QHideEvent *e);
  
private:
  void resetData();
  void doUndo(bool refresh);
  void doAction(bool preview);
  void doOnClose();
  void setColorTable(QRgb *ctab, int num);
  
  static const int ColorMode = 0;
  static const int BWMode = 1;
  
  ColorPalette *cp;
  QVBoxLayout *vbox;
  ApaintDoc *doc;
  bool previewMode, isOpen, enableShowColor;
  QString btnText;
  bool usedColors[256];
  int mode;
  int bwBorder;
  QFileDialog *fileDlg;
  IconViewDlg *iconDlg;
  QString palettePath, fileName;
};

/****************************ColNumDlg CLASS**********************************************/

class ColNumDlg : public  ColorNumDialogUI {
  Q_OBJECT
public:
  ColNumDlg(QWidget* parent, const char* name, ApaintDoc *docp);
  ~ColNumDlg();
  void startDlg();
  int getNumColors(){return maxCol;}

signals:
  void sendEvents(bool enable);

public slots:
  void layerChangedSlot(Layer*);
  void newDocSlot();

protected slots:
  void okBtnClicked();
  void cancelBtnClicked();
  void previewBtnClicked();
  void ditherchecked(bool enable);
  void valChanged(int val);

protected:
  void closeEvent(QCloseEvent *e);
  void hideEvent(QHideEvent *e);

private:
  void setVisualData(bool setSld);
  void updateVisual(Layer *layer);
  void showNoiseBox(bool enable);
  void doUndo(bool refresh);
  void doAction(bool preview);
  void doUpdate();
  void doOnClose();

  int ncol, maxCol;
  ApaintDoc *doc;
  bool isTrueCol;
  bool dlgOn;
  bool previewMode;
};


#endif
