/***************************************************************************
                          histogramdlg.cpp  -  description
                             -------------------
    begin                : Tue Nov 11 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dialogHistogram.h"
#include <qpushbutton.h>
#include <qpainter.h>
#include <qtabwidget.h>
#include <qlayout.h>
#include <apaintconfig.h>

Histogram::Histogram(QWidget *parent, const char *name):QScrollView(parent, name, WStaticContents|WResizeNoErase)
{
  allPix=0;
  for (int i=0; i<256; i++)
   histogramCol[i] = histogramNum[i] = 0; 
  dataLoaded = false;
  setHScrollBarMode(AlwaysOn);
  viewport()->setBackgroundMode(NoBackground);
}

void Histogram::setColorData(QRgb *ctab, int *hNumTab, int *hColTab)
{
 allPix = 0;
 numCol = 0;
 for (int i=0; i<256; i++)
 {
   colTab[i] = ctab[i];
   if (hColTab[i]) numCol++;
   histogramCol[i] = hColTab[i];
   histogramNum[i] = hNumTab[i];
   allPix+=hColTab[i];
 }
 dataLoaded = true;
 recalcData(); 
}

void Histogram::recalcData()
{
  if (!dataLoaded) return;
  QFont fnt = QApplication::font();
  QFontMetrics fm(font());
  wsize = QMAX(fm.width("100%(256)"), width() / numCol);
  hei= height() - (horizontalScrollBar()->height()+2);
  resizeContents(wsize*numCol, hei);
  repaintContents();
}

void Histogram::refresh(QPainter *p, QRect refRect)
{
 if (!dataLoaded) return;
 int maxCol, strWid, hsize;
 double perc;
 QString str;
 QRect textRect, barRect, whiteRect;

 QFontMetrics fm(font());
 maxCol = histogramCol[0]; 
 for (int i =0; i< 256; i++)
  {
   if (histogramCol[i]) /* there is color */
    {
     perc =(double)((int)(((double)histogramCol[i]/(double)allPix)*1000))/10;
     str = QString("%1(%2%)\n%3").arg(histogramNum[i] + ApaintConfig::getFirstPaletteColor()).arg(perc).arg(histogramCol[i]);
     strWid = fm.width(str);
     hsize = (int)QMAX(1, 50 + (hei-50)*(double)((double)histogramCol[i]/(double)maxCol)-fm.height()*2);
     textRect = QRect(wsize*i+((wsize-strWid)/2), hei-QMAX(hsize, fm.height()*2)-fm.height()*2, strWid, fm.height()*2);
     barRect = QRect(wsize*i, hei-hsize, wsize, hsize);
     whiteRect = QRect(wsize*i, 0, wsize, hei-hsize);
     if (refRect.intersects(whiteRect))
       p->fillRect(whiteRect.x(), whiteRect.y(), whiteRect.width(), whiteRect.height(), white);
     if (refRect.intersects(textRect))     
       p->drawText(textRect.x(), textRect.y(), textRect.width(), textRect.height(), AlignCenter, str);
     if (refRect.intersects(barRect))
       p->fillRect(barRect.x(), barRect.y(), barRect.width(), barRect.height(), QColor(colTab[histogramNum[i]]));
    }    
  }
}

void Histogram::viewportResizeEvent(QResizeEvent *e)
{
  QScrollView::viewportResizeEvent(e);
  recalcData();
}

void Histogram::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cliph)
{  
  QRect refRect = QRect(clipx, clipy, clipw, cliph);
  refresh(p, refRect);
}



/*HISTOGRAM 2*/

Histogram2::Histogram2(QWidget *parent, const char *name):QScrollView(parent, name, WRepaintNoErase|WResizeNoErase|WStaticContents)
{
  dataLoaded = false;
  viewport()->setBackgroundMode(NoBackground);
  setHScrollBarMode(AlwaysOn);
  imgLine = 0;
  scaleX = scaleY = 1;
}

Histogram2::~Histogram2()
{
 if (imgLine)
    delete[] imgLine;  
}

void Histogram2::setData(ApaintImage &img, double sx, double sy)
{
  if (img.isNull()) return;
  himg = img;
  dataLoaded = true;
  setScale(sx, sy);
}

void Histogram2::setScale(double sx, double sy)
{
  if (sx>1) sx = 1;
  if (sy>1) sy = 1;
  scaleX = sx;
  scaleY = sy;
  recalcData();
}

void Histogram2::recalcData()
{  

 int x, y, i, xs;
 uchar d;
 double hRatio;
 
 if (!dataLoaded) return;  
 if (imgLine)
    delete[] imgLine;

 wid =(int)(himg.width()*scaleX);
 hei = (int)(height()-(horizontalScrollBar()->height()+2));
 hRatio = ((double)hei/(double)himg.height());
 
 
 imgLine = new ColumnColors[wid+1];
 
 for (x = 0; x<wid; x++)
 {
  xs = (int)(x/scaleX);
  for (i=0; i<256; i++)
   imgLine[x].col[i] = 0;

  for (y=0; y<hei; y++)
  {
    d = *(himg.scanLine((int)(y/hRatio)%himg.height())+xs);
    imgLine[x].col[d]++;
  }
 }
 resizeContents(wid, hei);
 repaintContents(); 

}

void Histogram2::refresh(QPainter *p, QRect refRect)
{
 if (!dataLoaded) return;
 int yy, i, heiLine, l, r;
 
 l = 1;
 r = 0;
  
 if (refRect.left()<wid)
 {
   l = refRect.left();
   r =QMIN(refRect.right(), wid);
 }

 if (refRect.right()>wid)
  p->eraseRect(wid, 0, refRect.right()- wid, hei);
 
 for (int x=l; x<=r; x++)
 {  
     yy = 0;
     for (i = 0; i<256; i++)
     {
      if (imgLine[x].col[i])
      {        
        heiLine=imgLine[x].col[i];
        p->setPen(QColor(himg.color(i)));
        p->drawLine(x, yy, x, yy+heiLine);
        yy+=heiLine;
      }      
    }    
  }
}

void Histogram2::viewportResizeEvent(QResizeEvent *e)
{
 QScrollView::viewportResizeEvent(e);
 recalcData();
}      

void Histogram2::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cliph)
{
  QRect ref = QRect(clipx, clipy, clipw, cliph);
  refresh(p, ref);
}


/*HISTOGRAM 3*/

Histogram3::Histogram3(QWidget *parent, const char *name):QScrollView(parent, name, WRepaintNoErase|WResizeNoErase|WStaticContents)
{
  dataLoaded = false;
  viewport()->setBackgroundMode(NoBackground);
  setVScrollBarMode(AlwaysOn);
  imgLine = 0;
  scaleX = scaleY = 1;
}

Histogram3::~Histogram3()
{
 if (imgLine)
    delete[] imgLine;  
}

void Histogram3::setData(ApaintImage &img, double sx, double sy)
{
  if (img.isNull()) return;
  himg = img;
  dataLoaded = true;
  setScale(sx, sy);
}

void Histogram3::setScale(double sx, double sy)
{
  if (sx>1) sx = 1;
  if (sy>1) sy = 1;
  scaleX = sx;
  scaleY = sy;
  recalcData();
}

void Histogram3::recalcData()
{
 int x, y, i;
 uchar d;
 
 if (!dataLoaded) return;  
 if (imgLine)
    delete[] imgLine;

 wid = (int) (width()-(verticalScrollBar()->width()+2));
 hei = (int)(himg.height()*scaleY);

 imgLine = new ColumnColors[hei+1];
 for (y = 0; y<hei; y++)
 {
  for (i=0; i<256; i++)
   imgLine[y].col[i] = 0;

  for (x=0; x<wid; x++)
   {
    d = *(himg.scanLine((int)(y)%himg.width())+x);
    imgLine[y].col[d]++;
   }
 }
 resizeContents(wid, hei);
 repaintContents(); 
}

void Histogram3::refresh(QPainter *p, QRect refRect)
{
 if (!dataLoaded) return;
 int xx, i, widLine, t, b;



 
 t = 1;
 b = 0;
  
 if (refRect.top()<hei)
 {
   t = refRect.top();
   b =QMIN(refRect.bottom(), hei);
 }

 if (refRect.bottom()>hei)
  p->eraseRect(0, hei, wid, refRect.bottom() - hei);

/*
 l = 1;
 r = 0;
  
 if (refRect.left()<wid)
 {
   l = refRect.left();
   r =QMIN(refRect.right(), wid);
 }

 if (refRect.right()>wid)
  p->eraseRect(wid, 0, refRect.right()- wid, hei);
*/




 



 for (int y=t; y<=b; y++)
 {  
     xx = 0;
     for (i = 0; i<256; i++)
     {
      if (imgLine[y].col[i])
      {        
        widLine=imgLine[y].col[i];
        p->setPen(QColor(himg.color(i)));
        p->drawLine(xx, y, xx+widLine, y);
        xx+=widLine;
      }      
    }    
  }

/*
 for (int x=l; x<=r; x++)
 {  
     yy = 0;
     for (i = 0; i<256; i++)
     {
      if (imgLine[x].col[i])
      {        
        heiLine=imgLine[x].col[i];
        p->setPen(QColor(himg.color(i)));
        p->drawLine(x, yy, x, yy+heiLine);
        yy+=heiLine;
      }      
    }    
  }
*/

}

void Histogram3::viewportResizeEvent(QResizeEvent *e)
{
 QScrollView::viewportResizeEvent(e);
 recalcData();
}      

void Histogram3::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cliph)
{
  QRect ref = QRect(clipx, clipy, clipw, cliph);
  refresh(p, ref);
}

/*
    HISTOGRAM DIALOG
*/
HistogramDlg::HistogramDlg(QWidget* parent, const char* name, ApaintDoc *docp):QDialog(parent, name)
{
 Q_CHECK_PTR(docp) ;
 doc = docp;
 isOn = false;
 scaleX = scaleY = 1;
 resize( 390, 228 );
 setCaption( ApaintConfig::getText(647) );
 HistogramDialogLayout = new QVBoxLayout( this, 11, 6, "HistogramDialogLayout");

 tabHistogram = new QTabWidget( this, "tabHistogram" );
 
 Layout2 = new QHBoxLayout( 0, 0, 6, "Layout2");
 QSpacerItem* spacer = new QSpacerItem( 180, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
 Layout2->addItem( spacer );

 btnRefresh = new QPushButton( this, "btnRefresh" );
 btnRefresh->setText( ApaintConfig::getText(93) );
 Layout2->addWidget( btnRefresh );

 btnClose = new QPushButton( this, "btnClose" );
 btnClose->setText( ApaintConfig::getText(104));
 Layout2->addWidget( btnClose );
 HistogramDialogLayout->addWidget( tabHistogram );
 HistogramDialogLayout->addLayout( Layout2 );
 
 histo = new Histogram(this, "Histogram");
 histo2 = new Histogram2(this, "Histogram");
 histo3 = new Histogram3(this, "Histogram");
 tabHistogram->insertTab(histo, ApaintConfig::getText(770));
 tabHistogram->insertTab(histo2, ApaintConfig::getText(771));
 tabHistogram->insertTab(histo3, ApaintConfig::getText(772));
 connect(btnClose, SIGNAL(clicked()), SLOT(closeDlg()));
 connect(btnRefresh, SIGNAL(clicked()), SLOT(refreshDlg())); 
}

HistogramDlg::~HistogramDlg()
{
  
}

void HistogramDlg::startDlg()
{
  Layer *layer = doc->getLayer();
  if (!layer) return;
  
  if (layer->getImageType() != LOAD_CMAP)  return; /*just true col*/
  isOn = true;
  refreshHistogram();
  show();
}

void HistogramDlg::closeDlg()
{
  close();
  isOn = false;
}

void HistogramDlg::closeEvent(QCloseEvent *e)
{
  isOn = false;
  QDialog::closeEvent(e);
}

void HistogramDlg::refreshDlg()
{
 refreshHistogram(); 
}

void HistogramDlg::layerChanged(Layer *layer)
{
 if (isOn)
 {
  if (layer->getImageType() == LOAD_TRUE_COL)
  {
   close();
  }
  else
  {
    refreshHistogram();
  }
 }
}

void HistogramDlg::zoomChanged(double sx, double sy)
{
  scaleX = sx;
  scaleY = sy;
  if (isOn)
    histo2->setScale(scaleX, scaleY);
  if (isOn)
    histo3->setScale(scaleX, scaleY);
}

void HistogramDlg::refreshHistogram()
{
  int *hnum, *hcol;
  
  Layer *layer = doc->getLayer();
  layer->imageBuf.getHistogramData(&hnum, &hcol, layer->usingMask(), &layer->getMask());
  histo->setColorData(layer->imageBuf.colorTable(), hnum, hcol);
  histo2->setData(layer->imageBuf,scaleX, scaleY);
  histo3->setData(layer->imageBuf,scaleX, scaleY);
}

