/***************************************************************************
                          brushtool.cpp  -  description
                             -------------------
    begin                : pet feb 28 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "toolBrush.h"
#include "apaintdrawrepeat.h"
#include "apaintBrushList.h"
#include "apaintconfig.h"
#include "apaintdoc.h"
#include "qcheckbox.h"
#include "qlayout.h"
#include "apaintUtil.h"

extern char *getCommonPNG(const char *pixName);

BrushToolProp::BrushToolProp(QWidget *parent, const char *name):QWidget(parent, name)
{
  vbox = new QVBoxLayout(this);
  cbStamp  = new QCheckBox(ApaintConfig::getText(723), this);  
  cbStamp->setFocusPolicy(NoFocus);
  vbox->addWidget(cbStamp);
  connect(cbStamp, SIGNAL(toggled(bool)), this, SIGNAL(stampModeToggle(bool)));
}

BrushToolProp::~BrushToolProp()
{

}

BrushTool::BrushTool(DrawInRepeat *drawp, ToolType type, PropObject *propObj, ApaintDoc *docp):ToolBase(type, propObj)
{
 cur=QCursor(ArrowCursor);
 drawStarted=false;
 bList = ApaintBrushList::getObject();
 setToggleBtn(QPixmap(getCommonPNG("draw-freehand")), tr("Brush"));
 if (propObj)
 {
  setUsepWidget(true);
  brushProp = (BrushToolProp *) propObj->pWidget();
  connect(brushProp, SIGNAL(stampModeToggle(bool)), this, SLOT(enableStampMode(bool)));
 }
 cProp = CommonProp::getObject();
 if (cProp)
  connect(cProp, SIGNAL(setAspectDrawSig(bool)),this, SLOT(enableAspectDraw(bool)));
 Q_CHECK_PTR(drawp);
 draw = drawp;
 Q_CHECK_PTR(docp);
 doc = docp;
 setAutoScroll(false);
 stampMode=FALSE; /* since enableStampMode uses stampMode, it would be unititialised on first run */
 enableStampMode(false);
}

BrushTool::~BrushTool(){
}

void BrushTool::begin()
{
 setAutoScroll(false);
 if (brushProp->cbStamp->isChecked())
  enableStampMode(true);
}

void BrushTool::end()
{
 enableStampMode(false);
}

void BrushTool::showCommonProp(bool enable)
{
  CommonProp *cProp = CommonProp::getObject();
  cProp->setAspectDrawProp(enable);
  cProp->setThickLinesProp(enable); 
}

void BrushTool::enableStampMode(bool enable)
{  
  if (enable)
   {
    cProp->enableAspectDrawProp(false);
    bList->show();
   }
  else
    bList->hide();

  if (stampMode)
   doc->refreshLayer(bList->getPos());
  stampMode = enable;
}

void BrushTool::enableAspectDraw(bool enable)
{
  if (enable)
   brushProp->cbStamp->setChecked(false);
}

void BrushTool::drawBrush(QPainter *pi, int x, int y, bool final)
{
  if (!bList->isValid()) return;
  Layer *selLayer = doc->getLayer();
  QRect br = selLayer->layerPos();
  QBrush solidBrush =QBrush(Qt::SolidPattern);
  solidBrush.setColor(white);
  pi->setBrush(solidBrush);
  ApaintImage bimg = bList->getBrush();
  ApaintImage nullImg;
  if (bimg.isNull()) return;
  QImage mimg = bList->getBrushMask();
  bool useMask = !mimg.isNull();

  if (bList->isColorTabDifferent())
  {
    doc->putUndoStack(ChangeColorTable);
    bList->mergeColorTables(selLayer);
    emit colorTableChanged(selLayer);
  }
  
  QRect areaNorm = QRect(x-br.x()-bimg.width()/2, y-br.y()-bimg.height()/2, bimg.width(), bimg.height());
  QRect area = QRect(x-bimg.width()/2, y-bimg.height()/2, bimg.width(), bimg.height());
  draw->drawRectWithoutAspect(pi, area, false, false, false); //draw rect in pixmap for undo
  //origImage must be  null image so that we have data for Undo
  doc->copyImageToLayer(selLayer, areaNorm, bimg, nullImg, false, useMask, &mimg, 0, FALSE);
  //finalize stamp draw and put old data on undo stack
  draw->refreshDirect(area.x(), area.y(), area.width(), area.height(), false, false, final);
}

void BrushTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 if (drawStarted && btnPressed == RightButton) return;

 if (stampMode)
  {
   drawStarted = true;
   lastx=(int)(e->x()/zoomX);
   lasty=(int)(e->y()/zoomY);
   drawBrush(pi, lastx, lasty, false);
  }
 else
  {
   if (drawStarted && btnPressed == LeftButton)
    {
     drawStarted=false;
     draw->drawBrush(pi, lastx, lasty, lastx, lasty,  true, (btnPressed == LeftButton),  true);
    }
   btnPressed = e->button();
   if (btnPressed == MidButton) return;

   drawStarted=true;
   lastx=(int)(e->x()/zoomX);
   lasty=(int)(e->y()/zoomY);

   draw->drawBrush(pi, lastx, lasty, lastx, lasty, true, (btnPressed == LeftButton),  false);
  }
}

void BrushTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{ 
 
 if (stampMode)
 {
   int x = (int)((e->pos()).x()/zoomX);
   int y = (int)((e->pos()).y()/zoomY);
   int xx, yy;
   lastx = x;
   lasty = y;
   oldPos = bList->getPos();
   brushWid = bList->posWid();
   brushHei = bList->posHei();
   xx = x-brushWid/2;
   yy = y-brushHei/2;   
   pos = QRect(xx, yy, brushWid, brushHei);
   bList->setPos(pos);
   if (drawStarted)
    drawBrush(pi, lastx, lasty, false);
   else
    draw->refreshIntersectedArea(oldPos, pos, 0);
 }
 else
 {
   int x = (int)((e->pos()).x()/zoomX);
   int y = (int)((e->pos()).y()/zoomY);
   if (!drawStarted) return;
   draw->drawBrush(pi, lastx, lasty,x, y, true, (btnPressed == LeftButton),  false);
   lastx=x;
   lasty=y;
 }
}

void BrushTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 if (!drawStarted) return;
 if (stampMode)
 {
   drawBrush(pi, lastx, lasty, true);
   drawStarted = false;
 }
 else
 {
   drawStarted=false;
   draw->drawBrush(pi, lastx, lasty, lastx, lasty, true, (btnPressed == LeftButton),  true);
 }
}
