/***************************************************************************
                          recttool.cpp  -  description
                             -------------------
    begin                : pet feb 28 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "toolRect.h"
#include "apaintdrawrepeat.h"
#include "apaintconfig.h"
#include "apaintUtil.h"
#include "apaint.h"
#include "apaintdoc.h"

extern char *getCommonPNG(const char *pixName);


RectProp::RectProp(QWidget *parent, const char *name):QWidget(parent, name)
{
 QGridLayout *gbox = new QGridLayout(this, 4, 4);

 gbox->addWidget(new QLabel("X1:", this), 0, 0);
 gbox->addWidget(new QLabel("X2:", this), 0, 2);
 gbox->addWidget(new QLabel("Y1:", this), 1, 0);
 gbox->addWidget(new QLabel("Y2:", this), 1, 2);

 gbox->addMultiCellWidget(new QLabel(ApaintConfig::getText(WIDTH_MSG)+":", this), 2, 2, 0, 1);
 gbox->addMultiCellWidget(new QLabel(ApaintConfig::getText(HEIGHT_MSG)+":", this), 3, 3, 0, 1);

 rectOK =  new QPushButton(QPixmap(getCommonPNG("apply")), NULL, this);

 sbRectX1 = new QSpinBox(this);
 sbRectY1 = new QSpinBox(this);
 sbRectX2 = new QSpinBox(this);
 sbRectY2 = new QSpinBox(this);
 sbRectWid = new QSpinBox(this);
 sbRectHei = new QSpinBox(this);
 sbRectX1->setMaxValue(MAX_IMAGE);
 sbRectY1->setMaxValue(MAX_IMAGE);
 sbRectX2->setMaxValue(MAX_IMAGE);
 sbRectY2->setMaxValue(MAX_IMAGE);
 sbRectWid->setMaxValue(MAX_IMAGE);
 sbRectHei->setMaxValue(MAX_IMAGE);
 gbox->addWidget(sbRectX1, 0, 1);
 gbox->addWidget(sbRectX2, 0, 3);
 gbox->addWidget(sbRectY1, 1, 1);
 gbox->addWidget(sbRectY2, 1, 3);
 gbox->addWidget(sbRectWid, 2, 3);
 gbox->addWidget(sbRectHei, 3, 3);
 gbox->addMultiCellWidget(rectOK, 4, 4, 0, 3);
 connect(rectOK, SIGNAL(clicked()), this, SLOT(rectOKClicked()));

 cbFullX = new QCheckBox(ApaintConfig::getText(HORIZONTAL_MSG), this);
 cbFullX->setChecked(FALSE);
 cbFullY = new QCheckBox(ApaintConfig::getText(VERTICAL_MSG), this);
 cbFullY->setChecked(FALSE);
 gbox->addMultiCellWidget(cbFullX, 5, 5, 0, 1);
 gbox->addMultiCellWidget(cbFullY, 5, 5, 2, 3);
 connect(cbFullX, SIGNAL(toggled(bool)), this, SLOT(fullXToggled(bool)));
 connect(cbFullY, SIGNAL(toggled(bool)), this, SLOT(fullYToggled(bool)));
}

void RectProp::showFullXY(bool show)
{
 if (show)
  {
   cbFullX->show();
   cbFullY->show();
  }
 else
  {
   cbFullX->hide();
   cbFullY->hide();
  }
}

bool RectProp::isFullX() const
{
 return cbFullX->isChecked();
}

bool RectProp::isFullY() const
{
 return cbFullY->isChecked();
}

void RectProp::fullXToggled(bool set)
{
 if (set && cbFullY->isChecked())
  cbFullY->setChecked(FALSE);
}

void RectProp::fullYToggled(bool set)
{
 if (set && cbFullX->isChecked())
  cbFullX->setChecked(FALSE);
}

RectProp::~RectProp()
{
}

void RectProp::rectOKClicked()
{
 rTool->drawParametric();
}

RectTool::RectTool(DrawInRepeat *drawp, ApaintDoc *docp, ToolType type, RectType rtype, PropObject *propObj):ToolBase(type, propObj)
{
 cur=QCursor(ArrowCursor);
 drawStarted=false;
 drawType = NormalDraw;
 noBrush = QBrush(Qt::NoBrush);
 solidBrush = QBrush(Qt::SolidPattern);
 solidBrush.setColor(white);
 selBrush = noBrush;
 typ = rtype;
 if (typ == NormalRect)
  setToggleBtn(QPixmap(getCommonPNG("draw-rectangle")), QPixmap(getCommonPNG("draw-rectangle-filled")));
 else  if (typ == QuadRect)
  setToggleBtn(QPixmap(getCommonPNG("draw-square")), QPixmap(getCommonPNG("draw-square-filled")));
 else if (typ == EllipseRect)
  setToggleBtn(QPixmap(getCommonPNG("draw-ellipse")), QPixmap(getCommonPNG("draw-ellipse-filled")));
 else if (typ == CircleRect)
  setToggleBtn(QPixmap(getCommonPNG("draw-circle")), QPixmap(getCommonPNG("draw-circle-filled")));
 if (propObj)
  setUsepWidget(true);
 Q_CHECK_PTR(drawp); 
 draw = drawp;
 Q_CHECK_PTR(docp);
 doc = docp;
 setToolChangeType(FillChangeType);
 rProp=NULL;
 if (propObj)
  {
   rProp = (RectProp *) propObj->pWidget();
   rProp->setTool(this);
  }
 rProp->showFullXY(typ == NormalRect);
}

RectTool::~RectTool()
{
  
}

QPixmap RectTool::getToolPix()
{
 if (drawType == FillDraw)
  return toolPix2;

 return toolPix;
}

void RectTool::showCommonProp(bool enable)
{
 CommonProp *cProp = CommonProp::getObject();
 if (typ==NormalRect || typ==QuadRect)
  cProp->setThickLinesProp(false);
 else
  cProp->setThickLinesProp(enable);
 cProp->setAspectDrawProp(enable);
}


bool RectTool::doChangeTool(ToolChangeType chgType)
{
 if (chgType == FillChangeType)
  {    
   if (drawType == NormalDraw)
    {
     drawType = FillDraw;
     selBrush = solidBrush;
    }
   else
    {
     drawType = NormalDraw;
     selBrush = noBrush;
    }
   return true;
  }
 return false;
}

QRect RectTool::getRect(int x1, int y1, int x2, int y2)
{
 double ax = aspectX*doc->getAspectX();
 double ay = aspectY*doc->getAspectY();
  
 if (typ == NormalRect)
  return normalRect(x1, y1, x2, y2);
 else if (typ == QuadRect)
   return normalQuadRect(x1, y1, x2, y2, ax, ay);
 else if (typ == CircleRect)
    return normalCircle(x1, y1, x2, y2, ax, ay);
 else if (typ == EllipseRect)
    return normalEllipse(x1, y1, x2, y2);

 return QRect();
}

void RectTool::drawPrimitive(QPainter *pi, QRect r, bool doRefresh, bool useFg, bool final)
{
 if(typ == CircleRect || typ == EllipseRect)
  draw->drawEllipse(pi, r, doRefresh, useFg, final);
 else if (typ == NormalRect || typ == QuadRect)
  draw->drawRect(pi, r, doRefresh, useFg, final);
}

void RectTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 if (drawStarted) return;
 btnPressed = e->button();
 if (btnPressed!=LeftButton && btnPressed!=RightButton) return;

 drawStarted=true;
 startx=lastx=(int)(e->x()/zoomX);
 starty=lasty=(int)(e->y()/zoomY);

 if (rProp)
  {
   rProp->setRectX1(startx+1);
   rProp->setRectY1(doc->getMainLayer()->layerPos().height()-starty);
  }
}

void RectTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 if (!drawStarted) return;
 QRect rold, rnew;
 int x= (int)(e->x()/zoomX);
 int y= (int)(e->y()/zoomY);
 
 pi->setRasterOp(XorROP);
 pi->setBrush(selBrush);
 if (pi->brush().style() != NoBrush)
  pi->setPen(NoPen);

 if ((pi->pen().width()==0) && (typ == NormalRect || typ == QuadRect))
  pi->setPen(QPen(white, 1));

 drawPointer(pc, e->x(), e->y());

 if (rProp)
  {
   if (rProp->isFullX() && (typ == NormalRect) )
    {
     startx=0;
     x = doc->getLayer()->imageBuf.width();
    }
   else
   if (rProp->isFullY() && (typ == NormalRect) )
    {
     starty=0;
     y = doc->getLayer()->imageBuf.height();
    }
  }
   
 rnew = getRect(startx, starty, x, y);
 drawPrimitive(pi, rnew, true, (btnPressed == LeftButton), false);
 lastx=x;
 lasty=y;

 rold = getRect(startx, starty, lastx, lasty);
 drawPrimitive(pi, rold, false, false, false);
 drawPointer(pc, e->x(), e->y());

 if (rProp)
  {
   rProp->setRectX1(startx+1);
   rProp->setRectY1(doc->getMainLayer()->layerPos().height()-starty);
   rProp->setRectX2(lastx+1);
   rProp->setRectY2(doc->getMainLayer()->layerPos().height()-lasty);
   rProp->setRectWid(abs(startx-lastx)+1);
   rProp->setRectHei(abs(starty-lasty)+1);
  }
}

void RectTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 if (!drawStarted) return;
 QRect rold;
 drawStarted=false;

 pi->setRasterOp(CopyROP);
 pi->setBrush(selBrush);
 if (pi->brush().style() != NoBrush)
  pi->setPen(NoPen);

 if ((pi->pen().width()==0) && (typ == NormalRect || typ == QuadRect))
  pi->setPen(QPen(white, 1));
  
 rold = getRect(startx, starty, lastx, lasty);
 drawPrimitive(pi, rold, true, (btnPressed == LeftButton), true);
}

void RectTool::drawParametric()
{
 QPainter pi(doc->getLayer()->getPixmap());
 QRect rold;
 QPen pen = pi.pen();

 startx = rProp->getRectX1()-1;
 starty = doc->getMainLayer()->layerPos().height()-rProp->getRectY1();
 lastx = rProp->getRectX2()-1;
 lasty = doc->getMainLayer()->layerPos().height()-rProp->getRectY2();

// printf("clicked %d %d %d %d line width %d\n", startx, starty, lastx, lasty);

 pi.setRasterOp(CopyROP);
 pi.setBrush(selBrush);
 if (pi.brush().style() != NoBrush)
  pi.setPen(NoPen);
 pen.setWidth(lineWidth);
 pi.setPen(pen);

 if ((pi.pen().width()==0) && (typ == NormalRect || typ == QuadRect))
  pi.setPen(QPen(white, 1));

 rold = getRect(startx, starty, lastx, lasty);
 drawPrimitive(&pi, rold, true, true, true);
}
