/***************************************************************************
                          stitchtool.cpp  -  description
                             -------------------
    begin                : Thu Oct 16 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "toolStitch.h"
#include "apaintdrawrepeat.h"
#include "apaintdoc.h"
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include "apaintconfig.h"
#include <qlabel.h>

extern char *getCommonPNG(const char *pixName);

#define MAX 10000   /* max depth of stack */

typedef struct {short y, xl, xr, dy;} Segment;

#define PUSH(Y, XL, XR, DY) if ((Y+(DY)>=0) && (sp<stack+MAX) && (Y+(DY)<=hei)) \
    {sp->y = Y; sp->xl = XL; sp->xr = XR; sp->dy = DY; sp++; } /* push new segment on stack */

#define POP(Y, XL, XR, DY) /* pop segment off stack */ \
    {sp--; Y = sp->y+(DY = sp->dy); XL = sp->xl; XR = sp->xr;}

StitchPropWid::StitchPropWid(QWidget *parent, const char *name):StitchPropWidgetUI(parent, name)
{
  bgType->setTitle(ApaintConfig::getText(643));
  lbLen->setText(ApaintConfig::getText(642));
  rbLeft->setPixmap(QPixmap(getCommonPNG("go-previous")));
  rbRight->setPixmap(QPixmap(getCommonPNG("go-next")));
  rbLeftRight->setPixmap(QPixmap(getCommonPNG("shrink-horizontal")));
  rbLeftRight2->setPixmap(QPixmap(getCommonPNG("object-flip-horizontal")));
  rbUp->setPixmap(QPixmap(getCommonPNG("go-up")));
  rbDown->setPixmap(QPixmap(getCommonPNG("go-down")));
  rbUpDown->setPixmap(QPixmap(getCommonPNG("shrink-vertical")));
  rbUpDown2->setPixmap(QPixmap(getCommonPNG("object-flip-vertical")));
  cbStitchLine->setPixmap(QPixmap(getCommonPNG("check-16")));
}

int StitchPropWid::getDirect()
{
  QButton *d = (QRadioButton *) bgType->selected();
  if (d == rbLeft) return Left;
  if (d == rbRight) return Right;
  if (d == rbLeftRight) return LeftRight;
  if (d == rbLeftRight2) return LeftRight2;
  if (d == rbUp) return Up;
  if (d == rbDown) return Down;
  if (d == rbUpDown) return UpDown;
  if (d == rbUpDown2) return UpDown2;

  return Left;
}
                
StitchTool::StitchTool(DrawInRepeat *drawp, ApaintDoc *docp, ToolType type, PropObject *propObj):ToolBase(type, propObj)
{
 cur=QCursor(CrossCursor);
 setToggleBtn(QPixmap(getCommonPNG("stitch")), tr("Stitch Tool"));
 bgIndx=fgIndx=0;
 if (propObj)
 {
  setUsepWidget(true);
  stProp = (StitchPropWid *) propObj->pWidget();
 }
 Q_CHECK_PTR(drawp);
 Q_CHECK_PTR(docp);
 draw = drawp;
 doc = docp;
 protCol = 0;
 wasBtnPressed = false;
}

StitchTool::~StitchTool()
{
  
}

void StitchTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)                                                                                                  
{
 int x,y, i;
 int xmin, ymin, xmax, ymax;
 QRect fillRect;
 
 btnPressed = e->button();
 if (btnPressed == MidButton) return;

 Layer *selLayer = doc->getLayer();
 if (!selLayer) return;
 QRect br = selLayer->layerPos();
 wasBtnPressed = true;
 
 protCol = selLayer->getProtectColors(); 
 x=(int)(e->x()/zoomX)-br.x();
 y=(int)(e->y()/zoomY)-br.y();

 horPos = new MinMaxPos[br.width()];
 vertPos = new MinMaxHor[br.height()];
 for (i=0; i<br.height(); i++)
  vertPos[i].setAutoDelete(true);
  
 pi->translate(selLayer->drawAreaPos().x(), selLayer->drawAreaPos().y());
 imgPixMask = selLayer->getPixmap()->convertToImage();
 fillRect = fill(x,y, pi, selLayer->imageBuf, imgPixMask);

 if (fillRect.isValid())
  {
   xmin=fillRect.left()+br.x();
   ymin=fillRect.top()+br.y();
   xmax=fillRect.right()+br.x();
   ymax=fillRect.bottom()+br.y();

   draw->refreshDirect(xmin, ymin, QMAX(0,xmax-xmin), QMAX(0,ymax-ymin), (btnPressed == LeftButton), true, true);
  }
}

void StitchTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 Layer *selLayer = doc->getLayer();
 if (!selLayer) return;
 if (wasBtnPressed)
  {
    delete[] horPos;
    for (int i=0; i<selLayer->layerPos().height(); i++)
      vertPos[i].clear();
    delete[] vertPos;
    wasBtnPressed = false;
  }
}

bool StitchTool::validPoint(int xin, int yin, uint col, int colPtr, ApaintImage& img, QImage& imgPix)
{
 int x, y;
 QRect pos = doc->getLayerPos();
 if (!pos.contains(xin+pos.x(), yin+pos.y()))
  return false;
 
 if (doc->getLayer()->isUsingTiles())
  draw->transPoint(xin, yin, &x, &y);
 else
 {
   x = xin;
   y = yin;
 }

 if (PIX_ON(imgPix, x, y))
  return false;
  
 if (img.depth()>8)
  {
    if (col==img.pixel(x,y))
     return true;
    else
     return false;
  }
  else
  {
    if (colPtr==img.pixelIndex(x,y))
     return true;
    else
     return false;
  }
}

void StitchTool::setPixCol(int xin, int yin, QImage &imgPix)
{
  int x, y;

 if (doc->getLayer()->isUsingTiles())
  draw->transPoint(xin, yin, &x, &y);
 else
 {
   x = xin;
   y = yin;
 }

  imgPix.setPixel(x, y, 1);
}

void StitchTool::drawPoint(QPainter *p, int xin, int yin)
{
 int x, y;

 double ax = doc->getAspectX();
 double ay = doc->getAspectY();
 
 if (doc->getLayer()->isUsingTiles())
  draw->transPoint(xin, yin, &x, &y);
 else
 {
   x = xin;
   y = yin;
 }

 p->drawPoint((int)(x*ax), (int)(y*ay));
}

QRect StitchTool::fill(int x, int y, QPainter *p, ApaintImage& img, QImage& imgPix)
{
 int l, x1, x2, dy, wid, hei,xmin,ymin,xmax,ymax, xs;
 Segment stack[MAX], *sp = stack;	/* stack of filled segments */
 QRect pos = doc->getLayerPos();
 uint col;
 int colPtr, xin, yin;

 colPtr = col = 0;
 wid =pos.width();
 hei =pos.height();

 xmin= wid;
 ymin= hei;
 xmax=ymax=0;

 if (doc->getLayer()->isUsingTiles())
    draw->transPoint(x, y, &xin, &yin);
 else
  {
   xin = x;
   yin = y;
  }

 for (int i = 0; i < pos.width(); i++)
  {
   horPos[i].left = hei;
   horPos[i].right = 0;
  }

 if (img.depth()>8)
  {
   col=img.pixel(xin,yin);  /* read pv at seed point */
  }
 else
  {
   colPtr=img.pixelIndex(xin,yin);
  }

 if (x<0 || x>wid || y<0 || y>hei) return QRect();
 PUSH(y, x, x, 1);         /* needed in some cases */
 PUSH(y+1, x, x, -1);      /* seed segment (popped 1st) */

 while (sp>stack)
  {
   /* pop segment off stack and fill a neighboring scan line */
   POP(y, x1, x2, dy);
  /*
   * segment of scan line y-dy for x1<=x<=x2 was previously filled,
   * now explore adjacent pixels in scan line y
   */
  if (y>ymax) ymax=y;
  if (y<ymin) ymin=y;

  for (x=x1; x>=0 && validPoint(x,y, col, colPtr, img, imgPix); x--)
   {
    setPixCol(x, y, imgPix);
    setPosVal(&horPos[x], y);
   }
  if (y>=0 && y<hei)
   setPosVal(vertPos[y], x+1, x1);

  if (x<xmin) xmin=x;

  if (x>=x1)
   goto skip;
  l = x+1;
  if (l<x1) PUSH(y, l, x1-1, -dy);   /* leak on left? */
  x = x1+1;
  do
   {
    xs = x;
    for (; x<=wid && validPoint(x,y, col, colPtr, img, imgPix); x++)
    {
      setPixCol(x,y, imgPix);
      setPosVal(&horPos[x], y);
    }
    if (y>=0 && y<hei)
      setPosVal(vertPos[y], xs, x-1);

    if (x>xmax) xmax=x;
    PUSH(y, l, x-1, dy);
    if (x>x2+1) PUSH(y, x2+1, x-1, -dy);   /* leak on right? */
skip: for (x++; x<=x2 && (!validPoint(x,y, col, colPtr, img, imgPix)); x++) ;
       l = x;
   } while (x<=x2);
 }
 xmin=QMAX(0,xmin);
 ymin=QMAX(0,ymin);    
 p->eraseRect(xmin, ymin, QMAX(0,xmax-xmin), QMAX(0,ymax-ymin));

 switch (stProp->getDirect())
  {
   case StitchPropWid::Left:
    for (int ii = ymin; ii<ymax; ii++)
     clearLine(ii);
    drawLeft(ymin, ymax, p);
    break;
   case StitchPropWid::Right:
    for (int ii = ymin; ii<ymax; ii++)
     clearLine(ii);
    drawRight(ymin, ymax, p);
    break;
   case StitchPropWid::LeftRight:
    for (int ii = ymin; ii<ymax; ii++)
     clearLine(ii); 
    drawLeftRight(ymin, ymax, p);
    break;
   case StitchPropWid::LeftRight2:
    for (int ii = ymin; ii<ymax; ii++)
     clearLine(ii); 
    drawLeftRight2(ymin, ymax, p);
    break;
   case StitchPropWid::Up:
    drawUp(xmin, xmax, p);
    break;
   case StitchPropWid::Down:
    drawDown(xmin, xmax, p);
    break;
   case StitchPropWid::UpDown:
    drawUpDown(xmin, xmax, p);
    break;
   case StitchPropWid::UpDown2:
    drawUpDown2(xmin, xmax, p);
    break;
 }

//    return QRect(xmin, ymin, QMAX(0, xmax-xmin), QMAX(0, ymax-ymin)); was like this, but was returning one point too little
 return QRect(xmin, ymin, QMAX(0, xmax-xmin)+1, QMAX(0, ymax-ymin)+1);
}

void StitchTool::setPosVal(MinMaxPos *mm, int val)
{
  if (mm->left>=val)
    mm->left = val;
  if (mm->right<=val)
    mm->right = val;
}

void StitchTool::setPosVal(MinMaxHor& mm, int left, int right)
{
  MinMaxPos *pos = new MinMaxPos;
  pos->left = left;
  pos->right = right;
  mm.append(pos);
}

bool StitchTool::isPosIn(MinMaxHor& mm, int x)
{
  int i;
  MinMaxPos *pos;
  for (i=0; i<(int)mm.count(); i++)
  {
    pos = mm.at(i);
    if (x>=pos->left && x<=pos->right)
     return true;
  }
 return false;
}

void StitchTool::getSame(int posy, int &cnt)
{
  bool valid = true;
  MinMaxPos *pos, *nextPos;

  cnt = 0;
  for (int i=0; i<(int)vertPos[posy].count()-1 && valid; i++)
  {
    pos = vertPos[posy].at(i);
    nextPos = vertPos[posy].at(i+1);
    if (pos->right+1 == nextPos->left || pos->right == nextPos->left)
      cnt++;
    else
      valid = false;
  }
}

void StitchTool::clearLine(int ypos)
{
  int cnt, i, l, r;
  MinMaxPos *pos;
  MinMaxHor line;
  do
  {
    getSame(ypos, cnt);
    if (cnt>0)
    {
      pos = vertPos[ypos].at(0);
      l = pos->left;
      pos = vertPos[ypos].at(cnt);
      r = pos->right;
      pos = new MinMaxPos;
      pos->left = l;
      pos->right = r;
      line.append(pos);
      for (i=0; i<=cnt; i++)
        vertPos[ypos].remove((uint)0);
    }
    else
    {
      pos = vertPos[ypos].take(0);
      line.append(pos);
    }
  } while (vertPos[ypos].count());

  vertPos[ypos] = line;
}


void StitchTool::drawUp(int xmin, int xmax, QPainter *p)
{
 int x, y, t, d;
 int len = stProp->getLen();

 for (x= xmin; x<xmax; x++)
  {
   t = horPos[x].left+len;
   d = horPos[x].right;
   for (y = t; y<d; y+=len)
    if (isPosIn(vertPos[y], x))
     {
      if (stProp->doStitchLine() && ((x+y)&1)) 
       drawPoint(p, x, y+1);
      else
       drawPoint(p, x, y);
     }
  }
}

void StitchTool::drawDown(int xmin, int xmax, QPainter *p)
{
 int x, y, t, d;
 int len = stProp->getLen();

 for (x= xmin; x<xmax; x++)
  {
   t = horPos[x].left;
   d = horPos[x].right-len;

   for (y = d; y>t; y-=len)
    if (isPosIn(vertPos[y], x))
     {
      if (stProp->doStitchLine() && ((x+y)&1))
       drawPoint(p, x, y+1);
      else
       drawPoint(p, x, y);
     }
  }
}

void StitchTool::drawUpDown(int xmin, int xmax, QPainter *p)
{
 int x, t, d, center, fullLen;
 int len = stProp->getLen();

 for (x= xmin; x<xmax; x++)
  {
   t = horPos[x].left+len;
   d = horPos[x].right-len;
   
   fullLen = horPos[x].right-horPos[x].left;
   center = horPos[x].left + fullLen/2;

   if ( (fullLen>len) && ((fullLen%(len*2))>len) )
    if (isPosIn(vertPos[center], x))
     {
      if (stProp->doStitchLine() && ((x+center)&1))
       drawPoint(p, x, center+1);
      else
       drawPoint(p, x, center);
     }
 
   while (t<d)
    {
     if (isPosIn(vertPos[t], x))
      {
       if (stProp->doStitchLine() && ((x+t)&1))
	drawPoint(p, x, t+1);
       else
	drawPoint(p, x, t);
      }
     if (isPosIn(vertPos[d], x))
      {
       if (stProp->doStitchLine() && ((x+d)&1)) 
         drawPoint(p, x, d+1);
       else
         drawPoint(p, x, d);
      }
     t+=len;
     d-=len;
    }
  } 
}

void StitchTool::drawUpDown2(int xmin, int xmax, QPainter *p)
{
 int x, t, d, center, fullLen;
 int len = stProp->getLen();

 for (x= xmin; x<xmax; x++)
  {

   fullLen = horPos[x].right-horPos[x].left;
   center = horPos[x].left + fullLen/2;


   if (fullLen>len)
    if (isPosIn(vertPos[center], x))
     {
      if (stProp->doStitchLine() && ((x+center)&1))
       drawPoint(p, x, center+1);
      else
       drawPoint(p, x, center);
     }
   t = center-len;
   while (t>horPos[x].left)
    {
     if (isPosIn(vertPos[t], x))
      {
       if (stProp->doStitchLine() && ((x+t)&1))
	drawPoint(p, x, t+1);
       else
	drawPoint(p, x, t);
      }
     t-=len;
    }
   d = center+len;


   while (d<horPos[x].right)
    {
     if (isPosIn(vertPos[d], x))
      {
       if (stProp->doStitchLine() && ((x+d)&1)) 
        drawPoint(p, x, d+1);
       else
        drawPoint(p, x, d);
      }
     d+=len;
    }
  }
}
  
void StitchTool::drawLeft(int ymin, int ymax,  QPainter *p)
{
  int x, y, l, r, t, d, i;
  int len = stProp->getLen();
  MinMaxPos *pos;
  for (y = ymin; y<ymax; y++)
  {
   for (i=0; i<(int)vertPos[y].count(); i++)
   {     
     pos = vertPos[y].at(i);
     l = pos->left+len;
     r = pos->right;
   
     for (x = l; x < r; x+=len)
     {
      t = horPos[x].left;
      d = horPos[x].right;
      if (y>=t && y<=d)
      {
        if (stProp->doStitchLine() && ((x+y)&1))
          drawPoint(p, x+1, y);
        else
          drawPoint(p, x, y);
      }
     }
    }
  }
}

void StitchTool::drawRight(int ymin, int ymax, QPainter *p)
{
  int x, y, l, r, t, d, i;
  int len = stProp->getLen();
  MinMaxPos *pos;
  for (y = ymin; y<ymax; y++)
  {
   for (i=vertPos[y].count()-1; i>=0; i--)
   {
     pos = vertPos[y].at(i);
     l = pos->left;
     r = pos->right-len;

     for (x = r; x > l; x-=len)
     {
      t = horPos[x].left;
      d = horPos[x].right;
      if (y>=t && y<=d)
      {
        if (stProp->doStitchLine() && (x+y)&1)
          drawPoint(p, x+1, y);
        else
          drawPoint(p, x, y);
      }
     }
    }
  }
}

void StitchTool::drawLeftRight(int ymin, int ymax, QPainter *p)
{
 int  y, l, r, t, d, i, center, fullLen;
 int len = stProp->getLen();
 MinMaxPos *pos;

 for (y = ymin; y<ymax; y++)
  for (i=0; i<(int)vertPos[y].count(); i++)
   { 
    pos = vertPos[y].at(i);

    fullLen = pos->right - pos->left;
    center = pos->left + fullLen/2;
    if ( (fullLen>len) && ((fullLen%(len*2))>len) )
     {
      t = horPos[center].left;
      d = horPos[center].right;
      if (y>=t && y<=d)
       {
        if (stProp->doStitchLine() && (center+y)&1)
         drawPoint(p, center+1, y);
        else
         drawPoint(p, center, y);
       }
     }

    l = pos->left+len;
    r = pos->right-len;
    while (l<=r)
     {
      t = horPos[l].left;
      d = horPos[l].right;
      if (y>=t && y<=d)
       {
        if (stProp->doStitchLine() && (l+y)&1)
         drawPoint(p, l+1, y);
        else
         drawPoint(p, l, y);
       }
      t = horPos[r].left;
      d = horPos[r].right;
      if (y>=t && y<=d)
       {
        if (stProp->doStitchLine() && (r+y)&1)
         drawPoint(p, r+1, y);
        else
         drawPoint(p, r, y);
       }
      r-=len;
      l+=len;
     }
   }
}

void StitchTool::drawLeftRight2(int ymin, int ymax, QPainter *p)
{
 int  y, l, r, t, d, i, center;
 int len = stProp->getLen();
 MinMaxPos *pos;

 for (y = ymin; y<ymax; y++)
  for (i=0; i<(int)vertPos[y].count(); i++)
   { 
    pos = vertPos[y].at(i);

    if ((pos->right - pos->left)>len)
     {
      center = pos->left + (pos->right - pos->left)/2;
      t = horPos[center].left;
      d = horPos[center].right;
      if (y>=t && y<=d)
       {
        if (stProp->doStitchLine() && (center+y)&1)
         drawPoint(p, center+1, y);
        else
         drawPoint(p, center, y);
       }

      l = center-len;
      while (l>pos->left)
       {
	t = horPos[l].left;
	d = horPos[l].right;
	if (y>=t && y<=d)
	 {
          if (stProp->doStitchLine() && (l+y)&1)
           drawPoint(p, l+1, y);
          else
           drawPoint(p, l, y);
	 }
	l-=len;
       }
      r = center+len;
      while (r<pos->right)
       {
	t = horPos[r].left;
	d = horPos[r].right;
	if (y>=t && y<=d)
	 {
          if (stProp->doStitchLine() && (r+y)&1)
           drawPoint(p, r+1, y);
          else
           drawPoint(p, r, y);
	 }
	r+=len;
       }
     }
   }
}
