/***************************************************************************
                          toolsctrl.h  -  description
                             -------------------
    begin                : Sun Feb 2 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TOOLSCTRL_H
#define TOOLSCTRL_H

#include <qobject.h>
#include <toolbase.h>
#include <qrect.h>
#include <qpen.h>
#include "apaintimage.h"

class DrawInRepeat;
class ApaintDoc;
class Layer;

class ToolsCtrl : public QObject  {
  Q_OBJECT
public:

  ToolsCtrl(DrawInRepeat *drawRepIn, ApaintDoc *docp);
  ~ToolsCtrl();

  void registerTool(ToolBase *tool);
  void unregisterTool();
  ToolBase *getActiveTool();
  void getGlobalMousePos(int &xpo, int &ypos);
  void enableToolProp(PropObject *pobj, bool enable);
  
signals:

   void setCur(QCursor &cur);
   void unsetCur();
   void scrollBySig(int dx, int dy);
   void setColorSig(QRgb rgb);
   void setColorSigIndx(int indx);
   void setAutoScroll(bool On);
   void coordsPos(int x, int y);
   void drawStarted();
   void curRGB(uchar r, uchar g, uchar b, uchar indx);
   void clearSize();
    
public slots:

  void mouseMoveSlot(QMouseEvent *e, QPainter *pc);
  void mousePressSlot(QMouseEvent *e, QPainter *pc);
  void mouseReleaseSlot(QMouseEvent *e, QPainter *pc);
  void mouseDoubleClickSlot(QMouseEvent *e, QPainter *pc);
  void keyPressSlot(QKeyEvent *e, QPainter *pc);
  void keyReleaseSlot(QKeyEvent *e, QPainter *pc);
  void leaveEventSlot(QEvent* e, QPainter *pc);
  void enterEventSlot(QEvent* e, QPainter *pc);
  void beforeRefreshSlot(QPainter* pc);
  void afterRefreshSlot(QPainter* pc);
  void slotZoomChanged(double zx, double zy);
  void slotAspectChanged(double ax, double ay);
  void brushChgSlot(const QBrush &b);
  void lineSizeChgSlot(const QPen &p);
  void setSendEvents(bool on);
  void refreshPointer(QPainter *p);
  void repaintSlot(QPainter *pc);

   
private:

  void setImgPainter(QPainter *p);
  void setColor(int xs, int ys);
  void getColor(int xs, int ys);
  
  ToolBase *selectedTool;
  bool toolIsActive;
  double zoomX, zoomY;
  double aspectX, aspectY;
  QBrush mainBrush;
  QPen mainPen;
  QCursor colSel;
  int scrollStartX, scrollStartY;
  bool dragStart, shiftPressed;
  Layer *selLayer;
  bool sendEvents;
  bool toolIsBeingChanged;
  bool usingMask;
  int globX, globY;
  int gxpos, gypos;
  DrawInRepeat *drawRep;
  bool changePointer;
  int pointerPosX, pointerPosY;
  ApaintDoc *doc;
};

#endif
