#if !defined(__MINGW32__) && !defined(__APPLE__)
#define GIMPP
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "print.h"

static FILE *fout;
static char *fname;

/* the supported classes of printers; in variable printerClass */
#define CLASS_PHOTO_870	0
#define CLASS_PHOTO_890	1
#define CLASS_C80	2
#define CLASS_PHOTO_830	3
#define CLASS_PHOTO_950	4
#define CLASS_R200	5
#define CLASS_PHOTO_2100 6
#define CLASS_FAKE	7

static int printerClass = 0;
static int horDpi;

static unsigned short *blockLines;
static int numPrintBlocks;

static double global_c_level;
static double global_c_gamma = 1.0;
static double global_m_level;
static double global_m_gamma = 1.0;
static double global_y_level;
static double global_y_gamma = 1.0;
static double global_k_gamma = 1.0;
static double global_gamma = 1.0;
static double ink_limit;
static double density;
static char printer[100];
static char ink_type[30];
static char stp_resolution[50];
/* PhotoCMYK CMYK */
//char *media_source = "Standard";
static char media_type[50];
/* Plain Photo */
static char media_size[30];
/* Letter Legal A0 A1 A2 A3 A4 A5 */
//static char *dither_algorithm = "Ordered";
/* Adaptive Ordered Fast VeryFast Floyd */
static int printx, printy, offsetx, offsety;

static void fillCommon(printerMode *printerMod, int *numModes, printerDensity *prtDensityX,
 int numDensX, printerDensity *prtDensityY, int numDensY, int *dpiVariable, char **profileName)
{
 int i;

 printerMod[0].printType = IMG_TRUE_COL;
 printerMod[1].printType = IMG_BW;
 *numModes = 2;
 *dpiVariable = FALSE;
 for (i=1; i<numDensX; i++)
  {
   prtDensityX[i].dpi = prtDensityX[i-1].dpi/2;
   prtDensityX[i].pixels = prtDensityX[i-1].pixels/2;
  }
 for (i=1; i<numDensY; i++)
  {
   prtDensityY[i].dpi = prtDensityY[i-1].dpi/2;
   prtDensityY[i].pixels = prtDensityY[i-1].pixels/2;
  }
 switch (printerClass)
  {
   case CLASS_PHOTO_870: *profileName="StylusPhoto4"; break;
   case CLASS_PHOTO_890: *profileName="StylusPhoto5"; break;
   case CLASS_C80: *profileName="StylusC80"; break;
   case CLASS_PHOTO_830: *profileName="StylusPhoto830"; break;
   case CLASS_PHOTO_950: *profileName="StylusPhoto950"; break;
   case CLASS_R200: *profileName="StylusR200"; break;
   case CLASS_PHOTO_2100: *profileName="StylusPhoto2100"; break;
   case CLASS_FAKE: *profileName="screen"; break;
  }
}

/* return -1 if process can't be created, 0 on failure, 1 on success */
static int executeCommand(char *command)
{
 int status;

 status = system(command);
 if(status<0)
  return(-1);
 if (WIFEXITED(status))
  if (WEXITSTATUS(status)==0)
   return(TRUE);
  else
   return(FALSE);
 else
  return(FALSE);
}

static int initPrint(int printType, int complexImage, int xpixels,
 int ypixels, int xdpi, int ydpi, int xmargin, int ymargin)
{
 int filedes = -1;

 fname = strdup("/tmp/prnXXXXXX");
 if ((filedes=mkstemp(fname)) < 0 )
  return(PRT_IO_ERROR);
 else
  fout = fdopen(filedes, "wb");

 horDpi = xdpi/1000;
 printx = xpixels;
 printy = ypixels;
 offsetx = xmargin;
 offsety = ymargin;
 blockLines = calloc(sizeof(unsigned short), ypixels/2); /* assume at least 2 pixels per block */
 if (!blockLines)
  return(PRT_INTERNAL_ERROR);
 numPrintBlocks=0;
 return(PRT_OK);
}

static int printBand(int xpixels, char *data, int bpl, int ypos,
	int lines, char bandType, unsigned char *cmap)
{
 printBlock prtBlock;

 prtBlock.bandType=bandType;
 prtBlock.unreadLines=lines;
 prtBlock.bpl=bpl;
 prtBlock.ypos=ypos;
 if (bandType==IMG_CMAP)
  memcpy(&prtBlock.cmap, cmap, 256*3);
 blockLines[numPrintBlocks]=lines;

 /* first write printBlock structure */
 /* no danger for source/binary compatibility, since the file will always be written */
 /* and read by the same program. */
 if (!fwrite(&prtBlock, 1, sizeof(printBlock), fout))
  return(PRT_IO_ERROR);

 /* then the actual image block data */
 if (lines!=fwrite(data, bpl, lines, fout))
  return(PRT_IO_ERROR);

 numPrintBlocks++;
 return(PRT_OK);
}

static void writeParamFile(FILE *fp)
{
 int i;

 if (printerClass==CLASS_PHOTO_870)
  {
   if (horDpi==1440)
    {
     global_c_level = global_m_level = global_y_level = 1.3;
     ink_limit = 1.1;
     density = 1.3;
    }
   else /* 720 and 360 dpi settings */
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 1.0;
     density = 1.0;
    }
  }

 if (printerClass==CLASS_PHOTO_890)
  {
   if (horDpi==1440)
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 1.2;
     density = 1.2;
    }
   else /* 720 and 360 dpi settings */
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 1.0;
     density = 1.0;
    }
  }

 if (printerClass==CLASS_C80)
  {
   if (horDpi==2880)
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 1.0;
     density = 1.0;
    }
   else
    {
     global_c_level = global_m_level = global_y_level = 1.4;
     ink_limit = 1.1;
     density = 1.1;
    }
  }

 if (printerClass==CLASS_PHOTO_830)
  {
   global_c_level = global_m_level = global_y_level = 1.0;
   ink_limit = 1.0;
   density = 1.0;
  }

 if (printerClass==CLASS_PHOTO_950)
  {
   if (horDpi==1440)
    {
     global_c_level = global_m_level = global_y_level = 1.2;
     ink_limit = 1.1;
     density = 1.2;
    }
   else /* 720 and 360 dpi settings */
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 1.0;
     density = 1.0;
    }
  }

 if (printerClass==CLASS_R200)
  {
   if (horDpi==1440)
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 1.1;
     density = 1.2;
    }
   else /* 720 and 360 dpi settings */
    {
     global_c_level = global_m_level = global_y_level = 1.2;
     ink_limit = 1.1;
     density = 1.2;
    }
  }

 if (printerClass==CLASS_PHOTO_2100)
  {
   if (horDpi==1440)
    {
     global_c_level = global_m_level = global_y_level = 1.2;
     ink_limit = 1.1;
     density = 1.2;
    }
   else /* 720 and 360 dpi settings */
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 1.0;
     density = 1.0;
    }
  }

 if (printerClass==CLASS_FAKE)
  {
   global_c_level = global_m_level = global_y_level = 1.0;
   ink_limit = 1.0;
   density = 1.0;
  }

 if (ink_limit <= 0 || ink_limit > 2.0 ||
     global_c_level <= 0 || global_c_level > 10 ||
     global_m_level <= 0 || global_m_level > 10 ||
     global_y_level <= 0 || global_y_level > 10  )
  printf("Print options out of range!\n");


/* 360sw 360swuni */
/* 720sw 720swuni 720hquni */
/* 1440x720sw 1440x720swuni */
/* 2880x720sw  2880x720swuni 1440x720hq2 */
/* 1440x1440sw */
  if (horDpi==300)
   strcpy(stp_resolution, "300dpi");
  if (horDpi==600)
   strcpy(stp_resolution, "600dpi");
  if (horDpi==1200)
   strcpy(stp_resolution, "1200x1200dpi");
  if (horDpi==360)
   {
    if (strcmp(printer, "escp2-pro-xl")==0)
     strcpy(stp_resolution, "360mw");
    else
     strcpy(stp_resolution, "360sw");
   }
  if (horDpi==720)
   {
    if ((strcmp(printer, "escp2-9000")==0) || (strcmp(printer, "escp2-5500")==0) || (strcmp(printer, "escp2-pro-xl")==0))
     strcpy(stp_resolution, getBidirectionalPrinting() ? "720mw" : "720mwuni" );
    else
     strcpy(stp_resolution, getBidirectionalPrinting() ? "720sw" : "720swuni" );
   }
  if (horDpi==1440)
   strcpy(stp_resolution, getBidirectionalPrinting() ? "1440x720sw" : "1440x720swuni");
  if (horDpi==2880)
   strcpy(stp_resolution, getBidirectionalPrinting() ? "2880x720sw" : "2880x720swuni");

 if (printerClass==CLASS_FAKE)
  strcpy(media_type, "Plain");
 else
  strcpy(media_type, "Photo");

 /* start of writing parameters file */
 fprintf(fp, "%s\n", fname);
 fprintf(fp, "%s\n", getRegistredDstFile() ? getRegistredDstFile() : "");
 fprintf(fp, "%s\n", getRegistredCommandStr() ? getRegistredCommandStr() : "");
 fprintf(fp, "%d\n", getRegistredDoRemove());
 fprintf(fp, "%d %d %d %d\n", printx, printy, offsetx, offsety);
 fprintf(fp, "%d\n", getBidirectionalPrinting());
 fprintf(fp, "%f %f %f %f %f %f %f %f %f %f\n", global_c_level, global_c_gamma, global_m_level, global_m_gamma,  global_y_level, global_y_gamma, global_k_gamma, global_gamma, ink_limit, density);
 fprintf(fp, "%s\n", printer);
 fprintf(fp, "%s\n", ink_type);
 fprintf(fp, "%s\n", media_type);
 fprintf(fp, "%s\n", media_size);
 fprintf(fp, "%s\n", stp_resolution);
 fprintf(fp, "%d\n", horDpi);
 fprintf(fp, "%d\n", numPrintBlocks);
 for (i=0; i<numPrintBlocks; i++)
  fprintf(fp, "%d\n", blockLines[i]);
}

static int endPrint(int printStatus)
{
 if (printStatus==PRT_OK)
  {
   char *pfname;
   FILE *pfout;
   char command[300];
   int status;
   int filedes = -1;

   fflush(fout);
   fclose(fout);
   pfname = strdup("/tmp/prnXXXXXX");
   if ((filedes=mkstemp(pfname)) < 0 )
    { free(pfname); return(PRT_IO_ERROR); }
   else
    pfout = fdopen(filedes, "wb");

   writeParamFile(pfout);
   fclose(pfout);

   sprintf(command, "../common/aspawngp %s &", pfname);
   free(pfname);
   status = executeCommand(command);
   if (status<0)
    return(PRT_INTERNAL_ERROR);
   else
    return((status==0) ? PRT_IO_ERROR : PRT_OK);
  }
 else
  {
   fclose(fout);
   remove(fname);
   free(fname);
   fname=NULL;
  }
 return(printStatus);
}

static void getPrtInfoProXL(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 8300;
 prtDensityY[0].pixels = 11700;
 prtDensityX[0].dpi = 720000;
 prtDensityY[0].dpi = 720000;
 *numDensitX = 2;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusProLX (gimp-print)";
 strcpy(printer, "escp2-pro-xl");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A3");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoStylus3000(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11622;
 prtDensityY[0].pixels = 16355;
 prtDensityX[0].dpi = 720000;
 prtDensityY[0].dpi = 720000;
 *numDensitX = 2;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Stylus3000 (gimp-print)";
 strcpy(printer, "escp2-3000");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A2");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoStylusPro5500(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11622;
 prtDensityY[0].pixels = 16355;
 prtDensityX[0].dpi =  720000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 2;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusPro5500 (gimp-print)";
 strcpy(printer, "escp2-5500");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A3");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoStylus9000(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11622;
 prtDensityY[0].pixels = 16355;
 prtDensityX[0].dpi =  720000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 2;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Stylus9000 (gimp-print)";
 strcpy(printer, "escp2-9000");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A2");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPhoto870(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_PHOTO_870;
 prtDensityX[0].pixels = 11900;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusPhoto870 (gimp-print)";
 strcpy(printer, "escp2-870");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPhoto1270(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_PHOTO_870;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11700;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusPhoto1270 (gimp-print)";
 strcpy(printer, "escp2-1270");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A3");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPhoto890(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_PHOTO_890;
 prtDensityX[0].pixels = 11900;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusPhoto890 (gimp-print)";
 strcpy(printer, "escp2-890");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPhoto1290(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_PHOTO_890;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11700;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusPhoto1290 (gimp-print)";
 strcpy(printer, "escp2-1290");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A3");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoC70(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_C80;
 prtDensityX[0].pixels = 11520;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusC70 (gimp-print)";
 strcpy(printer, "escp2-c70");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoC80(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_C80;
 prtDensityX[0].pixels = 11520;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusC80 (gimp-print)";
 strcpy(printer, "escp2-c80");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoStylus670(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11520;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Stylus670 (gimp-print)";
 strcpy(printer, "escp2-670");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoStylus1160(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11700;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "Stylus1160 (gimp-print)";
 strcpy(printer, "escp2-1160");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A3");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoStylusCX3200(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11520;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusCX3200 (gimp-print)";
 strcpy(printer, "escp2-cx3200");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}


static void getPrtInfoPhoto830(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_PHOTO_830;
 prtDensityX[0].pixels = 11900;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusPhoto830 (gimp-print)";
 strcpy(printer, "escp2-830");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPhoto935(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_PHOTO_950;
 prtDensityX[0].pixels = 11900;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusPhoto935 (gimp-print)";
 strcpy(printer, "escp2-935");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPhoto950(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_PHOTO_950;
 prtDensityX[0].pixels = 11900;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusPhoto950 (gimp-print)";
 strcpy(printer, "escp2-950");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPhoto2100(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_PHOTO_2100;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11900;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusPhoto2100 (gimp-print)";
 strcpy(printer, "escp2-2100");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A3");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPhoto2200(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_PHOTO_2100;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11900;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusPhoto2200 (gimp-print)";
 strcpy(printer, "escp2-2200");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A3");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR200(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R200;
 prtDensityX[0].pixels = 11900;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusR200 (gimp-print)";
 strcpy(printer, "escp2-r200");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR300(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R200;
 prtDensityX[0].pixels = 11900;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusR300 (gimp-print)";
 strcpy(printer, "escp2-r300");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoRX500(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R200;
 prtDensityX[0].pixels = 11900;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusRX500 (gimp-print)";
 strcpy(printer, "escp2-rx500");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoRX600(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R200;
 prtDensityX[0].pixels = 11900;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusRX600 (gimp-print)";
 strcpy(printer, "escp2-rx600");
 strcpy(ink_type, "PhotoCMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoHPDeskjet1100C(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 4800;
 prtDensityY[0].pixels = 6600;
 prtDensityX[0].dpi =  600000;
 prtDensityY[0].dpi =  600000;
 *numDensitX = 2;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "HP Deskjet 1100C (gimp-print)";
 strcpy(printer, "pcl-1100");
// strcpy(ink_type, "CMYK");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}


static void getPrtInfoHPDeskjet1120C(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 /* this area is for A3 size */
 prtDensityX[0].pixels = 3300;
 prtDensityY[0].pixels = 4537;
 prtDensityX[0].dpi =  300000;
 prtDensityY[0].dpi =  300000;
 *numDensitX = 2;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "HP Deskjet 1120C (gimp-print)";
 strcpy(printer, "pcl-1120");
// strcpy(ink_type, "CMYK");
 strcpy(ink_type, "");
 strcpy(media_size, "A3");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}


static void getPrtInfoHPDeskjet1220C(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 6600;
 prtDensityY[0].pixels = 9074;
 prtDensityX[0].dpi =  600000;
 prtDensityY[0].dpi =  600000;
 *numDensitX = 2;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "HP Deskjet 1220C (gimp-print)";
 strcpy(printer, "pcl-1220");
// strcpy(ink_type, "CMYK");
 strcpy(ink_type, "");
 strcpy(media_size, "A3");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}


static void getPrtInfoHPDeskJet850C(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 /* this area is for A3 size */
 prtDensityX[0].pixels = 3300;
 prtDensityY[0].pixels = 4537;
 prtDensityX[0].dpi =  300000;
 prtDensityY[0].dpi =  300000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "HP Deskjet 850C (gimp-print)";
 strcpy(printer, "pcl-850");
// strcpy(ink_type, "CMYK");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}


static void getPrtInfoHPCP1700(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 /* this area is for A3 size */
 prtDensityX[0].pixels = 6600;
 prtDensityY[0].pixels = 9075;
 prtDensityX[0].dpi =  600000;
 prtDensityY[0].dpi =  600000;
 *numDensitX = 2;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "HP CP1700 (gimp-print)";
 strcpy(printer, "pcl-900");
 strcpy(ink_type, "");
// strcpy(media_size, "A3");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoStylus880(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11520;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Stylus880 (gimp-print)";
 strcpy(printer, "escp2-880");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoStylusC64(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11520;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusC64 (gimp-print)";
 strcpy(printer, "escp2-c64");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoStylusC82(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11520;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusC82 (gimp-print)";
 strcpy(printer, "escp2-c82");
 strcpy(ink_type, "CMYK");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

DGIMPP void registerStylusProXLGP(void)
{ registerPrinter(*getPrtInfoProXL, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylus3000GP(void)
{ registerPrinter(*getPrtInfoStylus3000, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusPro5500GP(void)
{ registerPrinter(*getPrtInfoStylusPro5500, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylus9000GP(void)
{ registerPrinter(*getPrtInfoStylus9000, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusPhoto870GP(void)
{ registerPrinter(*getPrtInfoPhoto870, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusPhoto1270GP(void)
{ registerPrinter(*getPrtInfoPhoto1270, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusPhoto890GP(void)
{ registerPrinter(*getPrtInfoPhoto890, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusPhoto1290GP(void)
{ registerPrinter(*getPrtInfoPhoto1290, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusC70GP(void)
{ registerPrinter(*getPrtInfoC70, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusC80GP(void)
{ registerPrinter(*getPrtInfoC80, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylus670GP(void)
{ registerPrinter(*getPrtInfoStylus670, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylus1160GP(void)
{ registerPrinter(*getPrtInfoStylus1160, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusCX3200GP(void)
{ registerPrinter(*getPrtInfoStylusCX3200, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusPhoto830GP(void)
{ registerPrinter(*getPrtInfoPhoto830, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusPhoto935GP(void)
{ registerPrinter(*getPrtInfoPhoto935, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusPhoto950GP(void)
{ registerPrinter(*getPrtInfoPhoto950, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusPhoto2100GP(void)
{ registerPrinter(*getPrtInfoPhoto2100, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusPhoto2200GP(void)
{ registerPrinter(*getPrtInfoPhoto2200, *initPrint, *printBand, *endPrint); }

DGIMPP void registerHPDeskJet1100CGP(void)
{ registerPrinter(*getPrtInfoHPDeskjet1100C, *initPrint, *printBand, *endPrint); }

DGIMPP void registerHPDeskJet1120CGP(void)
{ registerPrinter(*getPrtInfoHPDeskjet1120C, *initPrint, *printBand, *endPrint); }

DGIMPP void registerHPDeskJet1220CGP(void)
{ registerPrinter(*getPrtInfoHPDeskjet1220C, *initPrint, *printBand, *endPrint); }

DGIMPP void registerHPDeskJet850CGP(void)
{ registerPrinter(*getPrtInfoHPDeskJet850C, *initPrint, *printBand, *endPrint); }

DGIMPP void registerHPCP1700CGP(void)
{ registerPrinter(*getPrtInfoHPCP1700, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylus880GP(void)
{ registerPrinter(*getPrtInfoStylus880, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusC64GP(void)
{ registerPrinter(*getPrtInfoStylusC64, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusC82GP(void)
{ registerPrinter(*getPrtInfoStylusC82, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusR200GP(void)
{ registerPrinter(*getPrtInfoR200, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusR300GP(void)
{ registerPrinter(*getPrtInfoR300, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusRX500GP(void)
{ registerPrinter(*getPrtInfoRX500, *initPrint, *printBand, *endPrint); }

DGIMPP void registerStylusRX600GP(void)
{ registerPrinter(*getPrtInfoRX600, *initPrint, *printBand, *endPrint); }

#endif
